@@ -7,17 +7,22 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
 var (
-	pointerFile    string
-	pointerCompare string
-	pointerStdin   bool
+	pointerFile     string
+	pointerCompare  string
+	pointerStdin    bool
+	pointerCheck    bool
+	pointerStrict   bool
+	pointerNoStrict bool
 )
 
 func pointerCommand(cmd *cobra.Command, args []string) {
@@ -26,6 +31,43 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 	buildOid := ""
 	compareOid := ""
 
+	if pointerCheck {
+		var r io.ReadCloser
+		var err error
+
+		if pointerStrict && pointerNoStrict {
+			ExitWithError(errors.New(tr.Tr.Get("Cannot combine --strict with --no-strict")))
+		}
+
+		if len(pointerCompare) > 0 {
+			ExitWithError(errors.New(tr.Tr.Get("Cannot combine --check with --compare")))
+		}
+
+		if len(pointerFile) > 0 {
+			if pointerStdin {
+				ExitWithError(errors.New(tr.Tr.Get("With --check, --file cannot be combined with --stdin")))
+			}
+			r, err = os.Open(pointerFile)
+			if err != nil {
+				ExitWithError(err)
+			}
+		} else if pointerStdin {
+			r = ioutil.NopCloser(os.Stdin)
+		} else {
+			ExitWithError(errors.New(tr.Tr.Get("Must specify either --file or --stdin with --compare")))
+		}
+
+		p, err := lfs.DecodePointer(r)
+		if err != nil {
+			os.Exit(1)
+		}
+		if pointerStrict && !p.Canonical {
+			os.Exit(2)
+		}
+		r.Close()
+		return
+	}
+
 	if len(pointerCompare) > 0 || pointerStdin {
 		comparing = true
 	}
@@ -48,7 +90,7 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 		}
 
 		ptr := lfs.NewPointer(hex.EncodeToString(oidHash.Sum(nil)), size, nil)
-		fmt.Fprintf(os.Stderr, "Git LFS pointer for %s\n\n", pointerFile)
+		fmt.Fprint(os.Stderr, tr.Tr.Get("Git LFS pointer for %s", pointerFile), "\n\n")
 		buf := &bytes.Buffer{}
 		lfs.EncodePointer(io.MultiWriter(os.Stdout, buf), ptr)
 
@@ -58,7 +100,7 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 				Error(err.Error())
 				os.Exit(1)
 			}
-			fmt.Fprintf(os.Stderr, "\nGit blob OID: %s\n\n", buildOid)
+			fmt.Fprint(os.Stderr, "\n", tr.Tr.Get("Git blob OID: %s", buildOid), "\n\n")
 		}
 	} else {
 		comparing = false
@@ -81,7 +123,7 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 		if !pointerStdin {
 			pointerName = pointerCompare
 		}
-		fmt.Fprintf(os.Stderr, "Pointer from %s\n\n", pointerName)
+		fmt.Fprint(os.Stderr, tr.Tr.Get("Pointer from %s", pointerName), "\n\n")
 
 		if err != nil {
 			Error(err.Error())
@@ -95,17 +137,17 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 				Error(err.Error())
 				os.Exit(1)
 			}
-			fmt.Fprintf(os.Stderr, "\nGit blob OID: %s\n", compareOid)
+			fmt.Fprint(os.Stderr, "\n", tr.Tr.Get("Git blob OID: %s", compareOid), "\n")
 		}
 	}
 
 	if comparing && buildOid != compareOid {
-		fmt.Fprintf(os.Stderr, "\nPointers do not match\n")
+		fmt.Fprint(os.Stderr, "\n", tr.Tr.Get("Pointers do not match"), "\n")
 		os.Exit(1)
 	}
 
 	if !something {
-		Error("Nothing to do!")
+		Error(tr.Tr.Get("Nothing to do!"))
 		os.Exit(1)
 	}
 }
@@ -113,13 +155,13 @@ func pointerCommand(cmd *cobra.Command, args []string) {
 func pointerReader() (io.ReadCloser, error) {
 	if len(pointerCompare) > 0 {
 		if pointerStdin {
-			return nil, errors.New("Cannot read from STDIN and --pointer.")
+			return nil, errors.New(tr.Tr.Get("cannot read from STDIN and --pointer"))
 		}
 
 		return os.Open(pointerCompare)
 	}
 
-	requireStdin("The --stdin flag expects a pointer file from STDIN.")
+	requireStdin(tr.Tr.Get("The --stdin flag expects a pointer file from STDIN."))
 
 	return os.Stdin, nil
 }
@@ -129,5 +171,8 @@ func init() {
 		cmd.Flags().StringVarP(&pointerFile, "file", "f", "", "Path to a local file to generate the pointer from.")
 		cmd.Flags().StringVarP(&pointerCompare, "pointer", "p", "", "Path to a local file containing a pointer built by another Git LFS implementation.")
 		cmd.Flags().BoolVarP(&pointerStdin, "stdin", "", false, "Read a pointer built by another Git LFS implementation through STDIN.")
+		cmd.Flags().BoolVarP(&pointerCheck, "check", "", false, "Check whether the given file is a Git LFS pointer.")
+		cmd.Flags().BoolVarP(&pointerStrict, "strict", "", false, "Check whether the given Git LFS pointer is canonical.")
+		cmd.Flags().BoolVarP(&pointerNoStrict, "no-strict", "", false, "Don't check whether the given Git LFS pointer is canonical.")
 	})
 }
