@@ -2,39 +2,48 @@ package commands
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"os"
+	"runtime"
 	"sync"
 	"time"
 
-	"github.com/git-lfs/git-lfs/config"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/localstorage"
-	"github.com/git-lfs/git-lfs/progress"
-	"github.com/git-lfs/git-lfs/tools"
-	"github.com/git-lfs/git-lfs/tools/humanize"
-	"github.com/git-lfs/git-lfs/tq"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/fs"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tasklog"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tools/humanize"
+	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 	"github.com/spf13/cobra"
+	"golang.org/x/sync/semaphore"
 )
 
 var (
 	pruneDryRunArg      bool
 	pruneVerboseArg     bool
 	pruneVerifyArg      bool
+	pruneRecentArg      bool
+	pruneForceArg       bool
 	pruneDoNotVerifyArg bool
 )
 
 func pruneCommand(cmd *cobra.Command, args []string) {
 	// Guts of this must be re-usable from fetch --prune so just parse & dispatch
 	if pruneVerifyArg && pruneDoNotVerifyArg {
-		Exit("Cannot specify both --verify-remote and --no-verify-remote")
+		Exit(tr.Tr.Get("Cannot specify both --verify-remote and --no-verify-remote"))
 	}
 
-	fetchPruneConfig := cfg.FetchPruneConfig()
+	fetchPruneConfig := lfs.NewFetchPruneConfig(cfg.Git)
 	verify := !pruneDoNotVerifyArg &&
 		(fetchPruneConfig.PruneVerifyRemoteAlways || pruneVerifyArg)
+	fetchPruneConfig.PruneRecent = pruneRecentArg || pruneForceArg
+	fetchPruneConfig.PruneForce = pruneForceArg
 	prune(fetchPruneConfig, verify, pruneDryRunArg, pruneVerboseArg)
 }
 
@@ -53,18 +62,24 @@ type PruneProgress struct {
 }
 type PruneProgressChan chan PruneProgress
 
-func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbose bool) {
-	localObjects := make([]localstorage.Object, 0, 100)
+func prune(fetchPruneConfig lfs.FetchPruneConfig, verifyRemote, dryRun, verbose bool) {
+	localObjects := make([]fs.Object, 0, 100)
 	retainedObjects := tools.NewStringSetWithCapacity(100)
+
+	logger := tasklog.NewLogger(OutputWriter,
+		tasklog.ForceProgress(cfg.ForceProgress()),
+	)
+	defer logger.Close()
+
 	var reachableObjects tools.StringSet
 	var taskwait sync.WaitGroup
 
 	// Add all the base funcs to the waitgroup before starting them, in case
 	// one completes really fast & hits 0 unexpectedly
 	// each main process can Add() to the wg itself if it subdivides the task
-	taskwait.Add(4) // 1..4: localObjects, current & recent refs, unpushed, worktree
+	taskwait.Add(5) // 1..5: localObjects, current & recent refs, unpushed, worktree, stashes
 	if verifyRemote {
-		taskwait.Add(1) // 5
+		taskwait.Add(1) // 6
 	}
 
 	progressChan := make(PruneProgressChan, 100)
@@ -82,13 +97,18 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 	// Now find files to be retained from many sources
 	retainChan := make(chan string, 100)
 
-	gitscanner := lfs.NewGitScanner(nil)
-	go pruneTaskGetRetainedCurrentAndRecentRefs(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait)
-	go pruneTaskGetRetainedUnpushed(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait)
-	go pruneTaskGetRetainedWorktree(gitscanner, retainChan, errorChan, &taskwait)
+	gitscanner := lfs.NewGitScanner(cfg, nil)
+	gitscanner.Filter = filepathfilter.New(nil, cfg.FetchExcludePaths(), filepathfilter.GitIgnore)
+
+	sem := semaphore.NewWeighted(int64(runtime.NumCPU() * 2))
+
+	go pruneTaskGetRetainedCurrentAndRecentRefs(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait, sem)
+	go pruneTaskGetRetainedUnpushed(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait, sem)
+	go pruneTaskGetRetainedWorktree(gitscanner, fetchPruneConfig, retainChan, errorChan, &taskwait, sem)
+	go pruneTaskGetRetainedStashed(gitscanner, retainChan, errorChan, &taskwait, sem)
 	if verifyRemote {
 		reachableObjects = tools.NewStringSetWithCapacity(100)
-		go pruneTaskGetReachableObjects(gitscanner, &reachableObjects, errorChan, &taskwait)
+		go pruneTaskGetReachableObjects(gitscanner, &reachableObjects, errorChan, &taskwait, sem)
 	}
 
 	// Now collect all the retained objects, on separate wait
@@ -99,7 +119,7 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 	// Report progress
 	var progresswait sync.WaitGroup
 	progresswait.Add(1)
-	go pruneTaskDisplayProgress(progressChan, &progresswait)
+	go pruneTaskDisplayProgress(progressChan, &progresswait, logger)
 
 	taskwait.Wait() // wait for subtasks
 	gitscanner.Close()
@@ -116,7 +136,7 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 	var verifyQueue *tq.TransferQueue
 	var verifiedObjects tools.StringSet
 	var totalSize int64
-	var verboseOutput bytes.Buffer
+	var verboseOutput []string
 	var verifyc chan *tq.Transfer
 	var verifywait sync.WaitGroup
 
@@ -145,8 +165,11 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 			prunableObjects = append(prunableObjects, file.Oid)
 			totalSize += file.Size
 			if verbose {
-				// Save up verbose output for the end, spinner still going
-				verboseOutput.WriteString(fmt.Sprintf(" * %v (%v)\n", file.Oid, humanize.FormatBytes(uint64(file.Size))))
+				// Save up verbose output for the end.
+				verboseOutput = append(verboseOutput,
+					fmt.Sprintf("%s (%s)",
+						file.Oid,
+						humanize.FormatBytes(uint64(file.Size))))
 			}
 
 			if verifyRemote {
@@ -162,7 +185,7 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 	if verifyRemote {
 		verifyQueue.Wait()
 		verifywait.Wait()
-		close(progressChan) // after verify (uses spinner) but before check
+		close(progressChan) // after verify but before check
 		progresswait.Wait()
 		pruneCheckVerified(prunableObjects, reachableObjects, verifiedObjects)
 	} else {
@@ -171,22 +194,30 @@ func prune(fetchPruneConfig config.FetchPruneConfig, verifyRemote, dryRun, verbo
 	}
 
 	if len(prunableObjects) == 0 {
-		Print("Nothing to prune")
 		return
 	}
-	if dryRun {
-		Print("%d files would be pruned (%v)", len(prunableObjects), humanize.FormatBytes(uint64(totalSize)))
-		if verbose {
-			Print(verboseOutput.String())
-		}
-	} else {
-		Print("Pruning %d files, (%v)", len(prunableObjects), humanize.FormatBytes(uint64(totalSize)))
-		if verbose {
-			Print(verboseOutput.String())
-		}
-		pruneDeleteFiles(prunableObjects)
-	}
 
+	info := tasklog.NewSimpleTask()
+	logger.Enqueue(info)
+	if dryRun {
+		info.Logf("prune: %s", tr.Tr.GetN(
+			"%d file would be pruned (%s)",
+			"%d files would be pruned (%s)",
+			len(prunableObjects),
+			len(prunableObjects),
+			humanize.FormatBytes(uint64(totalSize))))
+		for _, item := range verboseOutput {
+			info.Logf("\n * %s", item)
+		}
+		info.Complete()
+	} else {
+		for _, item := range verboseOutput {
+			info.Logf("\n%s", item)
+		}
+		info.Complete()
+
+		pruneDeleteFiles(prunableObjects, logger)
+	}
 }
 
 func pruneCheckVerified(prunableObjects []string, reachableObjects, verifiedObjects tools.StringSet) {
@@ -208,23 +239,27 @@ func pruneCheckVerified(prunableObjects []string, reachableObjects, verifiedObje
 	// deleted but that's incorrect; bad state has occurred somehow, might need
 	// push --all to resolve
 	if problems.Len() > 0 {
-		Exit("Abort: these objects to be pruned are missing on remote:\n%v", problems.String())
+		Exit("%s\n%v", tr.Tr.Get("These objects to be pruned are missing on remote:"), problems.String())
 	}
 }
 
 func pruneCheckErrors(taskErrors []error) {
 	if len(taskErrors) > 0 {
 		for _, err := range taskErrors {
-			LoggedError(err, "Prune error: %v", err)
+			LoggedError(err, tr.Tr.Get("Prune error: %v", err))
 		}
-		Exit("Prune sub-tasks failed, cannot continue")
+		Exit(tr.Tr.Get("Prune sub-tasks failed, cannot continue"))
 	}
 }
 
-func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGroup) {
+func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGroup, logger *tasklog.Logger) {
 	defer waitg.Done()
 
-	spinner := progress.NewSpinner()
+	task := tasklog.NewSimpleTask()
+	defer task.Complete()
+
+	logger.Enqueue(task)
+
 	localCount := 0
 	retainCount := 0
 	verifyCount := 0
@@ -238,13 +273,14 @@ func pruneTaskDisplayProgress(progressChan PruneProgressChan, waitg *sync.WaitGr
 		case PruneProgressTypeVerify:
 			verifyCount++
 		}
-		msg = fmt.Sprintf("%d local objects, %d retained", localCount, retainCount)
+		msg = fmt.Sprintf("prune: %s, %s",
+			tr.Tr.GetN("%d local object", "%d local objects", localCount, localCount),
+			tr.Tr.GetN("%d retained", "%d retained", retainCount, retainCount))
 		if verifyCount > 0 {
-			msg += fmt.Sprintf(", %d verified with remote", verifyCount)
+			msg += tr.Tr.GetN(", %d verified with remote", ", %d verified with remote", verifyCount, verifyCount)
 		}
-		spinner.Print(OutputWriter, msg)
+		task.Log(msg)
 	}
-	spinner.Finish(OutputWriter, msg)
 }
 
 func pruneTaskCollectRetained(outRetainedObjects *tools.StringSet, retainChan chan string,
@@ -268,48 +304,55 @@ func pruneTaskCollectErrors(outtaskErrors *[]error, errorChan chan error, errorw
 	}
 }
 
-func pruneDeleteFiles(prunableObjects []string) {
-	spinner := progress.NewSpinner()
+func pruneDeleteFiles(prunableObjects []string, logger *tasklog.Logger) {
+	task := logger.Percentage(fmt.Sprintf("prune: %s", tr.Tr.Get("Deleting objects")), uint64(len(prunableObjects)))
+
 	var problems bytes.Buffer
 	// In case we fail to delete some
 	var deletedFiles int
-	for i, oid := range prunableObjects {
-		spinner.Print(OutputWriter, fmt.Sprintf("Deleting object %d/%d", i, len(prunableObjects)))
-		mediaFile, err := lfs.LocalMediaPath(oid)
+	for _, oid := range prunableObjects {
+		mediaFile, err := cfg.Filesystem().ObjectPath(oid)
 		if err != nil {
-			problems.WriteString(fmt.Sprintf("Unable to find media path for %v: %v\n", oid, err))
+			problems.WriteString(tr.Tr.Get("Unable to find media path for %v: %v", oid, err))
+			problems.WriteRune('\n')
+			continue
+		}
+		if mediaFile == os.DevNull {
 			continue
 		}
 		err = os.Remove(mediaFile)
 		if err != nil {
-			problems.WriteString(fmt.Sprintf("Failed to remove file %v: %v\n", mediaFile, err))
+			problems.WriteString(tr.Tr.Get("Failed to remove file %v: %v", mediaFile, err))
+			problems.WriteRune('\n')
 			continue
 		}
 		deletedFiles++
+		task.Count(1)
 	}
-	spinner.Finish(OutputWriter, fmt.Sprintf("Deleted %d files", deletedFiles))
 	if problems.Len() > 0 {
-		LoggedError(fmt.Errorf("Failed to delete some files"), problems.String())
-		Exit("Prune failed, see errors above")
+		LoggedError(errors.New(tr.Tr.Get("failed to delete some files")), problems.String())
+		Exit(tr.Tr.Get("Prune failed, see errors above"))
 	}
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetLocalObjects(outLocalObjects *[]localstorage.Object, progChan PruneProgressChan, waitg *sync.WaitGroup) {
+func pruneTaskGetLocalObjects(outLocalObjects *[]fs.Object, progChan PruneProgressChan, waitg *sync.WaitGroup) {
 	defer waitg.Done()
 
-	localObjectsChan := lfs.ScanObjectsChan()
-	for f := range localObjectsChan {
-		*outLocalObjects = append(*outLocalObjects, f)
+	cfg.EachLFSObject(func(obj fs.Object) error {
+		*outLocalObjects = append(*outLocalObjects, obj)
 		progChan <- PruneProgress{PruneProgressTypeLocal, 1}
-	}
+		return nil
+	})
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetRetainedAtRef(gitscanner *lfs.GitScanner, ref string, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetRetainedAtRef(gitscanner *lfs.GitScanner, ref string, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
+	sem.Acquire(context.Background(), 1)
+	defer sem.Release(1)
 	defer waitg.Done()
 
-	err := gitscanner.ScanRef(ref, func(p *lfs.WrappedPointer, err error) {
+	err := gitscanner.ScanTree(ref, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
 			errorChan <- err
 			return
@@ -325,17 +368,19 @@ func pruneTaskGetRetainedAtRef(gitscanner *lfs.GitScanner, ref string, retainCha
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetPreviousVersionsOfRef(gitscanner *lfs.GitScanner, ref string, since time.Time, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetPreviousVersionsOfRef(gitscanner *lfs.GitScanner, ref string, since time.Time, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
+	sem.Acquire(context.Background(), 1)
+	defer sem.Release(1)
 	defer waitg.Done()
 
 	err := gitscanner.ScanPreviousVersions(ref, since, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
 			errorChan <- err
 			return
-		} else {
-			retainChan <- p.Oid
-			tracerx.Printf("RETAIN: %v via ref %v >= %v", p.Oid, ref, since)
 		}
+
+		retainChan <- p.Oid
+		tracerx.Printf("RETAIN: %v via ref %v >= %v", p.Oid, ref, since)
 	})
 
 	if err != nil {
@@ -345,7 +390,7 @@ func pruneTaskGetPreviousVersionsOfRef(gitscanner *lfs.GitScanner, ref string, s
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetRetainedCurrentAndRecentRefs(gitscanner *lfs.GitScanner, fetchconf config.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetRetainedCurrentAndRecentRefs(gitscanner *lfs.GitScanner, fetchconf lfs.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
 
 	// We actually increment the waitg in this func since we kick off sub-goroutines
@@ -358,48 +403,50 @@ func pruneTaskGetRetainedCurrentAndRecentRefs(gitscanner *lfs.GitScanner, fetchc
 		return
 	}
 	commits.Add(ref.Sha)
-	waitg.Add(1)
-	go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg)
+	if !fetchconf.PruneForce {
+		waitg.Add(1)
+		go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg, sem)
+	}
 
 	// Now recent
-	if fetchconf.FetchRecentRefsDays > 0 {
+	if !fetchconf.PruneRecent && fetchconf.FetchRecentRefsDays > 0 {
 		pruneRefDays := fetchconf.FetchRecentRefsDays + fetchconf.PruneOffsetDays
 		tracerx.Printf("PRUNE: Retaining non-HEAD refs within %d (%d+%d) days", pruneRefDays, fetchconf.FetchRecentRefsDays, fetchconf.PruneOffsetDays)
 		refsSince := time.Now().AddDate(0, 0, -pruneRefDays)
 		// Keep all recent refs including any recent remote branches
 		refs, err := git.RecentBranches(refsSince, fetchconf.FetchRecentRefsIncludeRemotes, "")
 		if err != nil {
-			Panic(err, "Could not scan for recent refs")
+			Panic(err, tr.Tr.Get("Could not scan for recent refs"))
 		}
 		for _, ref := range refs {
 			if commits.Add(ref.Sha) {
 				// A new commit
 				waitg.Add(1)
-				go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg)
+				go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg, sem)
 			}
 		}
 	}
 
 	// For every unique commit we've fetched, check recent commits too
 	// Only if we're fetching recent commits, otherwise only keep at refs
-	if fetchconf.FetchRecentCommitsDays > 0 {
+	if !fetchconf.PruneRecent && fetchconf.FetchRecentCommitsDays > 0 {
 		pruneCommitDays := fetchconf.FetchRecentCommitsDays + fetchconf.PruneOffsetDays
 		for commit := range commits.Iter() {
 			// We measure from the last commit at the ref
 			summ, err := git.GetCommitSummary(commit)
 			if err != nil {
-				errorChan <- fmt.Errorf("Couldn't scan commits at %v: %v", commit, err)
+				errorChan <- errors.New(tr.Tr.Get("couldn't scan commits at %v: %v", commit, err))
 				continue
 			}
 			commitsSince := summ.CommitDate.AddDate(0, 0, -pruneCommitDays)
 			waitg.Add(1)
-			go pruneTaskGetPreviousVersionsOfRef(gitscanner, commit, commitsSince, retainChan, errorChan, waitg)
+			go pruneTaskGetPreviousVersionsOfRef(gitscanner, commit, commitsSince, retainChan, errorChan, waitg, sem)
 		}
 	}
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetRetainedUnpushed(gitscanner *lfs.GitScanner, fetchconf config.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetRetainedUnpushed(gitscanner *lfs.GitScanner, fetchconf lfs.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
 
 	err := gitscanner.ScanUnpushed(fetchconf.PruneRemoteName, func(p *lfs.WrappedPointer, err error) {
@@ -418,12 +465,16 @@ func pruneTaskGetRetainedUnpushed(gitscanner *lfs.GitScanner, fetchconf config.F
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetRetainedWorktree(gitscanner *lfs.GitScanner, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetRetainedWorktree(gitscanner *lfs.GitScanner, fetchconf lfs.FetchPruneConfig, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
 
+	if fetchconf.PruneForce {
+		return
+	}
+
 	// Retain other worktree HEADs too
 	// Working copy, branch & maybe commit is different but repo is shared
-	allWorktreeRefs, err := git.GetAllWorkTreeHEADs(config.LocalGitStorageDir)
+	allWorktreeRefs, err := git.GetAllWorkTreeHEADs(cfg.LocalGitStorageDir())
 	if err != nil {
 		errorChan <- err
 		return
@@ -443,16 +494,38 @@ func pruneTaskGetRetainedWorktree(gitscanner *lfs.GitScanner, retainChan chan st
 			// Worktree is on a different commit
 			waitg.Add(1)
 			// Don't need to 'cd' to worktree since we share same repo
-			go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg)
+			go pruneTaskGetRetainedAtRef(gitscanner, ref.Sha, retainChan, errorChan, waitg, sem)
 		}
 	}
 }
 
 // Background task, must call waitg.Done() once at end
-func pruneTaskGetReachableObjects(gitscanner *lfs.GitScanner, outObjectSet *tools.StringSet, errorChan chan error, waitg *sync.WaitGroup) {
+func pruneTaskGetRetainedStashed(gitscanner *lfs.GitScanner, retainChan chan string, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
+	defer waitg.Done()
+
+	err := gitscanner.ScanStashed(func(p *lfs.WrappedPointer, err error) {
+		if err != nil {
+			errorChan <- err
+		} else {
+			retainChan <- p.Pointer.Oid
+			tracerx.Printf("RETAIN: %v stashed", p.Pointer.Oid)
+		}
+	})
+
+	if err != nil {
+		errorChan <- err
+		return
+	}
+}
+
+// Background task, must call waitg.Done() once at end
+func pruneTaskGetReachableObjects(gitscanner *lfs.GitScanner, outObjectSet *tools.StringSet, errorChan chan error, waitg *sync.WaitGroup, sem *semaphore.Weighted) {
 	defer waitg.Done()
 
 	err := gitscanner.ScanAll(func(p *lfs.WrappedPointer, err error) {
+		sem.Acquire(context.Background(), 1)
+		defer sem.Release(1)
+
 		if err != nil {
 			errorChan <- err
 			return
@@ -469,6 +542,8 @@ func init() {
 	RegisterCommand("prune", pruneCommand, func(cmd *cobra.Command) {
 		cmd.Flags().BoolVarP(&pruneDryRunArg, "dry-run", "d", false, "Don't delete anything, just report")
 		cmd.Flags().BoolVarP(&pruneVerboseArg, "verbose", "v", false, "Print full details of what is/would be deleted")
+		cmd.Flags().BoolVarP(&pruneRecentArg, "recent", "", false, "Prune even recent objects")
+		cmd.Flags().BoolVarP(&pruneForceArg, "force", "f", false, "Prune everything that has been pushed")
 		cmd.Flags().BoolVarP(&pruneVerifyArg, "verify-remote", "c", false, "Verify that remote has LFS files before deleting")
 		cmd.Flags().BoolVar(&pruneDoNotVerifyArg, "no-verify-remote", false, "Override lfs.pruneverifyremotealways and don't verify")
 	})
