@@ -2,57 +2,55 @@ package commands
 
 import (
 	"fmt"
+	"os"
 	"sync"
 	"time"
 
-	"github.com/git-lfs/git-lfs/filepathfilter"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
-	"github.com/git-lfs/git-lfs/progress"
-	"github.com/git-lfs/git-lfs/tq"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tasklog"
+	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 	"github.com/spf13/cobra"
 )
 
 func pullCommand(cmd *cobra.Command, args []string) {
 	requireGitVersion()
-	requireInRepo()
+	setupRepository()
 
-	var remote string
 	if len(args) > 0 {
 		// Remote is first arg
-		if err := git.ValidateRemote(args[0]); err != nil {
-			Panic(err, fmt.Sprintf("Invalid remote name '%v'", args[0]))
+		if err := cfg.SetValidRemote(args[0]); err != nil {
+			Exit(tr.Tr.Get("Invalid remote name %q: %s", args[0], err))
 		}
-		remote = args[0]
-	} else {
-		// Actively find the default remote, don't just assume origin
-		defaultRemote, err := git.DefaultRemote()
-		if err != nil {
-			Panic(err, "No default remote")
-		}
-		remote = defaultRemote
 	}
 
 	includeArg, excludeArg := getIncludeExcludeArgs(cmd)
-	filter := buildFilepathFilter(cfg, includeArg, excludeArg)
-	pull(remote, filter)
+	filter := buildFilepathFilter(cfg, includeArg, excludeArg, true)
+	pull(filter)
 }
 
-func pull(remote string, filter *filepathfilter.Filter) {
-	cfg.CurrentRemote = remote
+func pull(filter *filepathfilter.Filter) {
 	ref, err := git.CurrentRef()
 	if err != nil {
-		Panic(err, "Could not pull")
+		Panic(err, tr.Tr.Get("Could not pull"))
 	}
 
 	pointers := newPointerMap()
-	meter := progress.NewMeter(progress.WithOSEnv(cfg.Os))
-	singleCheckout := newSingleCheckout()
-	q := newDownloadQueue(singleCheckout.manifest, remote, tq.WithProgress(meter))
-	gitscanner := lfs.NewGitScanner(func(p *lfs.WrappedPointer, err error) {
+	logger := tasklog.NewLogger(os.Stdout,
+		tasklog.ForceProgress(cfg.ForceProgress()),
+	)
+	meter := tq.NewMeter(cfg)
+	meter.Logger = meter.LoggerFromEnv(cfg.Os)
+	logger.Enqueue(meter)
+	remote := cfg.Remote()
+	singleCheckout := newSingleCheckout(cfg.Git, remote)
+	q := newDownloadQueue(singleCheckout.Manifest(), remote, tq.WithProgress(meter))
+	gitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
-			LoggedError(err, "Scanner error: %s", err)
+			LoggedError(err, tr.Tr.Get("Scanner error: %s", err))
 			return
 		}
 
@@ -61,8 +59,8 @@ func pull(remote string, filter *filepathfilter.Filter) {
 		}
 
 		// no need to download objects that exist locally already
-		lfs.LinkOrCopyFromReference(p.Oid, p.Size)
-		if lfs.ObjectExistsOfSize(p.Oid, p.Size) {
+		lfs.LinkOrCopyFromReference(cfg, p.Oid, p.Size)
+		if cfg.LFSObjectExists(p.Oid, p.Size) {
 			singleCheckout.Run(p)
 			return
 		}
@@ -89,9 +87,8 @@ func pull(remote string, filter *filepathfilter.Filter) {
 	}()
 
 	processQueue := time.Now()
-	if err := gitscanner.ScanTree(ref.Sha); err != nil {
+	if err := gitscanner.ScanTree(ref.Sha, nil); err != nil {
 		singleCheckout.Close()
-
 		ExitWithError(err)
 	}
 
@@ -112,7 +109,11 @@ func pull(remote string, filter *filepathfilter.Filter) {
 	if !success {
 		c := getAPIClient()
 		e := c.Endpoints.Endpoint("download", remote)
-		Exit("error: failed to fetch some objects from '%s'", e.Url)
+		Exit(tr.Tr.Get("Failed to fetch some objects from '%s'", e.Url))
+	}
+
+	if singleCheckout.Skip() {
+		fmt.Println(tr.Tr.Get("Skipping object checkout, Git LFS is not installed for this repository.\nConsider installing it with 'git lfs install'."))
 	}
 }
 
