@@ -6,11 +6,13 @@ import (
 	"fmt"
 	"io"
 	"os"
-	"regexp"
+	"path/filepath"
 	"strings"
 
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfs"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfs"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
@@ -20,20 +22,22 @@ var (
 )
 
 func statusCommand(cmd *cobra.Command, args []string) {
-	requireInRepo()
+	setupWorkingCopy()
 
 	// tolerate errors getting ref so this works before first commit
 	ref, _ := git.CurrentRef()
 
 	scanIndexAt := "HEAD"
+	var err error
 	if ref == nil {
-		scanIndexAt = git.RefBeforeFirstCommit
+		scanIndexAt, err = git.EmptyTree()
+		if err != nil {
+			ExitWithError(err)
+		}
 	}
 
-	scanner, err := lfs.NewPointerScanner()
+	scanner, err := lfs.NewPointerScanner(cfg.GitEnv(), cfg.OSEnv())
 	if err != nil {
-		scanner.Close()
-
 		ExitWithError(err)
 	}
 
@@ -52,19 +56,31 @@ func statusCommand(cmd *cobra.Command, args []string) {
 		ExitWithError(err)
 	}
 
-	Print("\nGit LFS objects to be committed:\n")
+	wd, _ := os.Getwd()
+	repo := cfg.LocalWorkingDir()
+
+	wd = tools.ResolveSymlinks(wd)
+
+	Print("\n%s\n", tr.Tr.Get("Objects to be committed:"))
 	for _, entry := range staged {
+		// Find a path from the current working directory to the
+		// absolute path of each side of the entry.
+		src := relativize(wd, filepath.Join(repo, entry.SrcName))
+		dst := relativize(wd, filepath.Join(repo, entry.DstName))
+
 		switch entry.Status {
 		case lfs.StatusRename, lfs.StatusCopy:
-			Print("\t%s -> %s (%s)", entry.SrcName, entry.DstName, formatBlobInfo(scanner, entry))
+			Print("\t%s -> %s (%s)", src, dst, formatBlobInfo(scanner, entry))
 		default:
-			Print("\t%s (%s)", entry.SrcName, formatBlobInfo(scanner, entry))
+			Print("\t%s (%s)", src, formatBlobInfo(scanner, entry))
 		}
 	}
 
-	Print("\nGit LFS objects not staged for commit:\n")
+	Print("\n%s\n", tr.Tr.Get("Objects not staged for commit:"))
 	for _, entry := range unstaged {
-		Print("\t%s (%s)", entry.SrcName, formatBlobInfo(scanner, entry))
+		src := relativize(wd, filepath.Join(repo, entry.SrcName))
+
+		Print("\t%s (%s)", src, formatBlobInfo(scanner, entry))
 	}
 
 	Print("")
@@ -74,8 +90,6 @@ func statusCommand(cmd *cobra.Command, args []string) {
 	}
 }
 
-var z40 = regexp.MustCompile(`\^?0{40}`)
-
 func formatBlobInfo(s *lfs.PointerScanner, entry *lfs.DiffIndexEntry) string {
 	fromSha, fromSrc, err := blobInfoFrom(s, entry)
 	if err != nil {
@@ -98,7 +112,7 @@ func formatBlobInfo(s *lfs.PointerScanner, entry *lfs.DiffIndexEntry) string {
 
 func blobInfoFrom(s *lfs.PointerScanner, entry *lfs.DiffIndexEntry) (sha, from string, err error) {
 	var blobSha string = entry.SrcSha
-	if z40.MatchString(blobSha) {
+	if git.IsZeroObjectID(blobSha) {
 		blobSha = entry.DstSha
 	}
 
@@ -115,11 +129,11 @@ func blobInfoTo(s *lfs.PointerScanner, entry *lfs.DiffIndexEntry) (sha, from str
 }
 
 func blobInfo(s *lfs.PointerScanner, blobSha, name string) (sha, from string, err error) {
-	if !z40.MatchString(blobSha) {
+	if !git.IsZeroObjectID(blobSha) {
 		s.Scan(blobSha)
 		if err := s.Err(); err != nil {
 			if git.IsMissingObject(err) {
-				return "<missing>", "?", nil
+				return fmt.Sprintf("<%s>", tr.Tr.Get("missing")), "?", nil
 			}
 			return "", "", err
 		}
@@ -134,27 +148,35 @@ func blobInfo(s *lfs.PointerScanner, blobSha, name string) (sha, from string, er
 		return s.ContentsSha()[:7], from, nil
 	}
 
-	f, err := os.Open(name)
+	f, err := os.Open(filepath.Join(cfg.LocalWorkingDir(), name))
+	if os.IsNotExist(err) {
+		return tr.Tr.Get("deleted"), tr.Tr.Get("File"), nil
+	}
 	if err != nil {
 		return "", "", err
 	}
 	defer f.Close()
 
+	// We've replaced a file with a directory.
+	if fi, err := f.Stat(); err == nil && fi.Mode().IsDir() {
+		return tr.Tr.Get("deleted"), tr.Tr.Get("File"), nil
+	}
+
 	shasum := sha256.New()
 	if _, err = io.Copy(shasum, f); err != nil {
 		return "", "", err
 	}
 
-	return fmt.Sprintf("%x", shasum.Sum(nil))[:7], "File", nil
+	return fmt.Sprintf("%x", shasum.Sum(nil))[:7], tr.Tr.Get("File"), nil
 }
 
 func scanIndex(ref string) (staged, unstaged []*lfs.DiffIndexEntry, err error) {
-	uncached, err := lfs.NewDiffIndexScanner(ref, false)
+	uncached, err := lfs.NewDiffIndexScanner(ref, false, true)
 	if err != nil {
 		return nil, nil, err
 	}
 
-	cached, err := lfs.NewDiffIndexScanner(ref, true)
+	cached, err := lfs.NewDiffIndexScanner(ref, true, false)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -208,26 +230,26 @@ func statusScanRefRange(ref *git.Ref) {
 		return
 	}
 
-	Print("On branch %s", ref.Name)
+	Print(tr.Tr.Get("On branch %s", ref.Name))
 
-	remoteRef, err := git.CurrentRemoteRef()
+	remoteRef, err := cfg.GitConfig().CurrentRemoteRef()
 	if err != nil {
 		return
 	}
 
-	gitscanner := lfs.NewGitScanner(func(p *lfs.WrappedPointer, err error) {
+	gitscanner := lfs.NewGitScanner(cfg, func(p *lfs.WrappedPointer, err error) {
 		if err != nil {
-			Panic(err, "Could not scan for Git LFS objects")
+			Panic(err, tr.Tr.Get("Could not scan for Git LFS objects"))
 			return
 		}
 
-		Print("\t%s (%s)", p.Name)
+		Print("\t%s (%s)", p.Name, p.Oid)
 	})
 	defer gitscanner.Close()
 
-	Print("Git LFS objects to be pushed to %s:\n", remoteRef.Name)
-	if err := gitscanner.ScanRefRange(ref.Sha, "^"+remoteRef.Sha, nil); err != nil {
-		Panic(err, "Could not scan for Git LFS objects")
+	Print("%s\n", tr.Tr.Get("Objects to be pushed to %s:", remoteRef.Name))
+	if err := gitscanner.ScanRefRange(ref.Sha, remoteRef.Sha, nil); err != nil {
+		Panic(err, tr.Tr.Get("Could not scan for Git LFS objects"))
 	}
 
 }
@@ -311,6 +333,39 @@ func porcelainStatusLine(entry *lfs.DiffIndexEntry) string {
 	return fmt.Sprintf("%s  %s", entry.Status, entry.SrcName)
 }
 
+// relativize relatives a path from "from" to "to". For instance, note that, for
+// any paths "from" and "to", that:
+//
+//	to == filepath.Clean(filepath.Join(from, relativize(from, to)))
+func relativize(from, to string) string {
+	if len(from) == 0 {
+		return to
+	}
+
+	flist := strings.Split(filepath.ToSlash(from), "/")
+	tlist := strings.Split(filepath.ToSlash(to), "/")
+
+	var (
+		divergence int
+		min        int
+	)
+
+	if lf, lt := len(flist), len(tlist); lf < lt {
+		min = lf
+	} else {
+		min = lt
+	}
+
+	for ; divergence < min; divergence++ {
+		if flist[divergence] != tlist[divergence] {
+			break
+		}
+	}
+
+	return strings.Repeat("../", len(flist)-divergence) +
+		strings.Join(tlist[divergence:], "/")
+}
+
 func init() {
 	RegisterCommand("status", statusCommand, func(cmd *cobra.Command) {
 		cmd.Flags().BoolVarP(&porcelain, "porcelain", "p", false, "Give the output in an easy-to-parse format for scripts.")
