@@ -6,20 +6,14 @@ import (
 	"os"
 	"strings"
 
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
 // untrackCommand takes a list of paths as an argument, and removes each path from the
 // default attributes file (.gitattributes), if it exists.
 func untrackCommand(cmd *cobra.Command, args []string) {
-	if cfg.LocalGitDir() == "" {
-		Print("Not a git repository.")
-		os.Exit(128)
-	}
-	if cfg.LocalWorkingDir() == "" {
-		Print("This operation must be run in a work tree.")
-		os.Exit(128)
-	}
+	setupWorkingCopy()
 
 	installHooks(false)
 
@@ -37,7 +31,7 @@ func untrackCommand(cmd *cobra.Command, args []string) {
 
 	attributesFile, err := os.Create(".gitattributes")
 	if err != nil {
-		Print("Error opening .gitattributes for writing")
+		Print(tr.Tr.Get("Error opening '.gitattributes' for writing"))
 		return
 	}
 	defer attributesFile.Close()
@@ -55,7 +49,7 @@ func untrackCommand(cmd *cobra.Command, args []string) {
 
 		path := strings.Fields(line)[0]
 		if removePath(path, args) {
-			Print("Untracking %q", unescapeTrackPattern(path))
+			Print(tr.Tr.Get("Untracking %q", unescapeAttrPattern(path)))
 		} else {
 			attributesFile.WriteString(line + "\n")
 		}
@@ -63,8 +57,9 @@ func untrackCommand(cmd *cobra.Command, args []string) {
 }
 
 func removePath(path string, args []string) bool {
+	withoutCurrentDir := trimCurrentPrefix(path)
 	for _, t := range args {
-		if path == escapeTrackPattern(t) {
+		if withoutCurrentDir == escapeAttrPattern(trimCurrentPrefix(t)) {
 			return true
 		}
 	}
