@@ -9,7 +9,9 @@ import (
 	"sync"
 	"time"
 
-	"github.com/git-lfs/git-lfs/config"
+	"github.com/git-lfs/git-lfs/v3/config"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/spf13/cobra"
 )
 
@@ -55,17 +57,46 @@ func RegisterCommand(name string, runFn func(cmd *cobra.Command, args []string),
 // It returns an exit code.
 func Run() int {
 	log.SetOutput(ErrorWriter)
+	tr.InitializeLocale()
 
 	root := NewCommand("git-lfs", gitlfsCommand)
 	root.PreRun = nil
 
 	// Set up help/usage funcs based on manpage text
+	helpcmd := &cobra.Command{
+		Use:   "help [command]",
+		Short: "Help about any command",
+		Long: `Help provides help for any command in the application.
+Simply type ` + root.Name() + ` help [path to command] for full details.`,
+
+		Run: func(c *cobra.Command, args []string) {
+			cmd, _, e := c.Root().Find(args)
+			// In the case of "git lfs help config" or "git lfs help
+			// faq", pretend the last arg was "help" so our command
+			// lookup succeeds, since cmd will be ignored in
+			// helpCommand().
+			if e != nil && (args[0] == "config" || args[0] == "faq") {
+				cmd, _, e = c.Root().Find([]string{"help"})
+			}
+			if cmd == nil || e != nil {
+				c.Println(tr.Tr.Get("Unknown help topic %#q", args))
+				c.Root().Usage()
+			} else {
+				c.HelpFunc()(cmd, args)
+			}
+		},
+	}
+
+	root.SetHelpCommand(helpcmd)
+
 	root.SetHelpTemplate("{{.UsageString}}")
 	root.SetHelpFunc(helpCommand)
 	root.SetUsageFunc(usageCommand)
 
 	root.Flags().BoolVarP(&rootVersion, "version", "v", false, "")
 
+	canonicalizeEnvironment()
+
 	cfg = config.New()
 
 	for _, f := range commandFuncs {
@@ -104,10 +135,13 @@ func usageCommand(cmd *cobra.Command) error {
 }
 
 func printHelp(commandName string) {
+	if commandName == "--help" {
+		commandName = "git-lfs"
+	}
 	if txt, ok := ManPages[commandName]; ok {
-		fmt.Fprintf(os.Stdout, "%s\n", strings.TrimSpace(txt))
+		fmt.Println(strings.TrimSpace(txt))
 	} else {
-		fmt.Fprintf(os.Stdout, "Sorry, no usage text found for %q\n", commandName)
+		fmt.Println(tr.Tr.Get("Sorry, no usage text found for %q", commandName))
 	}
 }
 
@@ -117,15 +151,15 @@ func setupHTTPLogger(cmd *cobra.Command, args []string) {
 	}
 
 	logBase := filepath.Join(cfg.LocalLogDir(), "http")
-	if err := os.MkdirAll(logBase, 0755); err != nil {
-		fmt.Fprintf(os.Stderr, "Error logging http stats: %s\n", err)
+	if err := tools.MkdirAll(logBase, cfg); err != nil {
+		fmt.Fprintln(os.Stderr, tr.Tr.Get("Error logging HTTP stats: %s", err))
 		return
 	}
 
 	logFile := fmt.Sprintf("http-%d.log", time.Now().Unix())
 	file, err := os.Create(filepath.Join(logBase, logFile))
 	if err != nil {
-		fmt.Fprintf(os.Stderr, "Error logging http stats: %s\n", err)
+		fmt.Fprintln(os.Stderr, tr.Tr.Get("Error logging HTTP stats: %s", err))
 	} else {
 		getAPIClient().LogHTTPStats(file)
 	}
