@@ -16,6 +16,7 @@ import (
 	"github.com/git-lfs/git-lfs/v3/tasklog"
 	"github.com/git-lfs/git-lfs/v3/tools"
 	"github.com/git-lfs/git-lfs/v3/tq"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 )
 
@@ -31,42 +32,37 @@ func uploadForRefUpdates(ctx *uploadContext, updates []*git.RefUpdate, pushAll b
 	}()
 
 	verifyLocksForUpdates(ctx.lockVerifier, updates)
-	rightSides := make([]string, 0, len(updates))
+	exclude := make([]string, 0, len(updates))
 	for _, update := range updates {
-		right := update.Right().Sha
-		if update.LeftCommitish() != right {
-			rightSides = append(rightSides, right)
+		remoteRefSha := update.RemoteRef().Sha
+		if update.LocalRefCommitish() != remoteRefSha {
+			exclude = append(exclude, remoteRefSha)
 		}
 	}
 	for _, update := range updates {
 		// initialized here to prevent looped defer
 		q := ctx.NewQueue(
-			tq.RemoteRef(update.Right()),
+			tq.RemoteRef(update.RemoteRef()),
 		)
-		err := uploadLeftOrAll(gitscanner, ctx, q, rightSides, update, pushAll)
+		err := uploadRangeOrAll(gitscanner, ctx, q, exclude, update, pushAll)
 		ctx.CollectErrors(q)
 
 		if err != nil {
-			return errors.Wrap(err, fmt.Sprintf("ref %s:", update.Left().Name))
+			return errors.Wrap(err, tr.Tr.Get("ref %q:", update.LocalRef().Name))
 		}
 	}
 
 	return nil
 }
 
-func uploadLeftOrAll(g *lfs.GitScanner, ctx *uploadContext, q *tq.TransferQueue, bases []string, update *git.RefUpdate, pushAll bool) error {
+func uploadRangeOrAll(g *lfs.GitScanner, ctx *uploadContext, q *tq.TransferQueue, exclude []string, update *git.RefUpdate, pushAll bool) error {
 	cb := ctx.gitScannerCallback(q)
 	if pushAll {
-		if err := g.ScanRefWithDeleted(update.LeftCommitish(), cb); err != nil {
+		if err := g.ScanRefWithDeleted(update.LocalRefCommitish(), cb); err != nil {
 			return err
 		}
 	} else {
-		left := update.LeftCommitish()
-		right := update.Right().Sha
-		if left == right {
-			right = ""
-		}
-		if err := g.ScanMultiRangeToRemote(left, bases, cb); err != nil {
+		if err := g.ScanMultiRangeToRemote(update.LocalRefCommitish(), exclude, cb); err != nil {
 			return err
 		}
 	}
@@ -194,8 +190,8 @@ func (c *uploadContext) prepareUpload(unfiltered ...*lfs.WrappedPointer) []*lfs.
 	// scanner
 	uniqOids := tools.NewStringSet()
 
-	// separate out objects that _should_ be uploaded, but don't exist in
-	// .git/lfs/objects. Those will skipped if the server already has them.
+	// Skip any objects which we've seen or already uploaded, as well
+	// as any which are locked by other users.
 	for _, p := range unfiltered {
 		// object already uploaded in this process, or we've already
 		// seen this OID (see above), skip!
@@ -244,7 +240,7 @@ func (c *uploadContext) UploadPointers(q *tq.TransferQueue, unfiltered ...*lfs.W
 				continue
 			}
 
-			Print("push %s => %s", p.Oid, p.Name)
+			Print("%s %s => %s", tr.Tr.Get("push"), p.Oid, p.Name)
 			c.SetUploaded(p.Oid)
 		}
 
@@ -289,23 +285,25 @@ func (c *uploadContext) ReportErrors() {
 	if len(c.missing) > 0 || len(c.corrupt) > 0 {
 		var action string
 		if c.allowMissing {
-			action = "missing objects"
+			action = tr.Tr.Get("missing objects")
 		} else {
-			action = "failed"
+			action = tr.Tr.Get("failed")
 		}
 
-		Print("LFS upload %s:", action)
+		Print(tr.Tr.Get("Git LFS upload %s:", action))
 		for name, oid := range c.missing {
-			Print("  (missing) %s (%s)", name, oid)
+			// TRANSLATORS: Leading spaces should be preserved.
+			Print(tr.Tr.Get("  (missing) %s (%s)", name, oid))
 		}
 		for name, oid := range c.corrupt {
-			Print("  (corrupt) %s (%s)", name, oid)
+			// TRANSLATORS: Leading spaces should be preserved.
+			Print(tr.Tr.Get("  (corrupt) %s (%s)", name, oid))
 		}
 
 		if !c.allowMissing {
 			pushMissingHint := []string{
-				"hint: Your push was rejected due to missing or corrupt local objects.",
-				"hint: You can disable this check with: 'git config lfs.allowincompletepush true'",
+				tr.Tr.Get("hint: Your push was rejected due to missing or corrupt local objects."),
+				tr.Tr.Get("hint: You can disable this check with: `git config lfs.allowincompletepush true`"),
 			}
 			Print(strings.Join(pushMissingHint, "\n"))
 			os.Exit(2)
@@ -317,18 +315,18 @@ func (c *uploadContext) ReportErrors() {
 	}
 
 	if c.lockVerifier.HasUnownedLocks() {
-		Print("Unable to push locked files:")
+		Print(tr.Tr.Get("Unable to push locked files:"))
 		for _, unowned := range c.lockVerifier.UnownedLocks() {
 			Print("* %s - %s", unowned.Path(), unowned.Owners())
 		}
 
 		if c.lockVerifier.Enabled() {
-			Exit("ERROR: Cannot update locked files.")
+			Exit(tr.Tr.Get("Cannot update locked files."))
 		} else {
-			Error("WARNING: The above files would have halted this push.")
+			Error(tr.Tr.Get("warning: The above files would have halted this push."))
 		}
 	} else if c.lockVerifier.HasOwnedLocks() {
-		Print("Consider unlocking your own locked files: (`git lfs unlock <path>`)")
+		Print(tr.Tr.Get("Consider unlocking your own locked files: (`git lfs unlock <path>`)"))
 		for _, owned := range c.lockVerifier.OwnedLocks() {
 			Print("* %s", owned.Path())
 		}
@@ -357,9 +355,11 @@ func (c *uploadContext) uploadTransfer(p *lfs.WrappedPointer) (*tq.Transfer, err
 
 	localMediaPath, err := c.gitfilter.ObjectPath(oid)
 	if err != nil {
-		return nil, errors.Wrapf(err, "Error uploading file %s (%s)", filename, oid)
+		return nil, errors.Wrap(err, tr.Tr.Get("Error uploading file %s (%s)", filename, oid))
 	}
 
+	// Skip the object if its corresponding file does not exist in
+	// .git/lfs/objects/.
 	if len(filename) > 0 {
 		if missing, err = c.ensureFile(filename, localMediaPath, oid); err != nil && !errors.IsCleanPointerError(err) {
 			return nil, err
