@@ -1,9 +1,15 @@
 package errors
 
 import (
+	goerrors "errors"
 	"fmt"
 	"net/url"
+	"os/exec"
+	"strconv"
+	"syscall"
+	"time"
 
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/pkg/errors"
 )
 
@@ -88,6 +94,19 @@ func IsNotAPointerError(err error) bool {
 	return false
 }
 
+// IsNotAPointerError indicates the parsed data is not an LFS pointer.
+func IsPointerScanError(err error) bool {
+	if e, ok := err.(interface {
+		PointerScanError() bool
+	}); ok {
+		return e.PointerScanError()
+	}
+	if parent := parentOf(err); parent != nil {
+		return IsPointerScanError(parent)
+	}
+	return false
+}
+
 // IsBadPointerKeyError indicates that the parsed data has an invalid key.
 func IsBadPointerKeyError(err error) bool {
 	if e, ok := err.(interface {
@@ -102,6 +121,20 @@ func IsBadPointerKeyError(err error) bool {
 	return false
 }
 
+// IsProtocolError indicates that the SSH pkt-line protocol data is invalid.
+func IsProtocolError(err error) bool {
+	if e, ok := err.(interface {
+		ProtocolError() bool
+	}); ok {
+		return e.ProtocolError()
+	}
+
+	if parent := parentOf(err); parent != nil {
+		return IsProtocolError(parent)
+	}
+	return false
+}
+
 // If an error is abad pointer error of any type, returns NotAPointerError
 func StandardizeBadPointerError(err error) error {
 	if IsBadPointerKeyError(err) {
@@ -127,6 +160,20 @@ func IsDownloadDeclinedError(err error) bool {
 	return false
 }
 
+// IsDownloadDeclinedError indicates that the upload operation failed because of
+// an HTTP 422 response code.
+func IsUnprocessableEntityError(err error) bool {
+	if e, ok := err.(interface {
+		UnprocessableEntityError() bool
+	}); ok {
+		return e.UnprocessableEntityError()
+	}
+	if parent := parentOf(err); parent != nil {
+		return IsUnprocessableEntityError(parent)
+	}
+	return false
+}
+
 // IsRetriableError indicates the low level transfer had an error but the
 // caller may retry the operation.
 func IsRetriableError(err error) bool {
@@ -144,6 +191,18 @@ func IsRetriableError(err error) bool {
 	return false
 }
 
+func IsRetriableLaterError(err error) (time.Time, bool) {
+	if e, ok := err.(interface {
+		RetriableLaterError() (time.Time, bool)
+	}); ok {
+		return e.RetriableLaterError()
+	}
+	if parent := parentOf(err); parent != nil {
+		return IsRetriableLaterError(parent)
+	}
+	return time.Time{}, false
+}
+
 type errorWithCause interface {
 	Cause() error
 	StackTrace() errors.StackTrace
@@ -161,7 +220,7 @@ type wrappedError struct {
 // newWrappedError creates a wrappedError.
 func newWrappedError(err error, message string) *wrappedError {
 	if err == nil {
-		err = errors.New("Error")
+		err = errors.New(tr.Tr.Get("Error"))
 	}
 
 	var errWithCause errorWithCause
@@ -211,7 +270,7 @@ func (e fatalError) Fatal() bool {
 }
 
 func NewFatalError(err error) error {
-	return fatalError{newWrappedError(err, "Fatal error")}
+	return fatalError{newWrappedError(err, tr.Tr.Get("Fatal error"))}
 }
 
 // Definitions for IsNotImplementedError()
@@ -225,7 +284,7 @@ func (e notImplementedError) NotImplemented() bool {
 }
 
 func NewNotImplementedError(err error) error {
-	return notImplementedError{newWrappedError(err, "Not implemented")}
+	return notImplementedError{newWrappedError(err, tr.Tr.Get("Not implemented"))}
 }
 
 // Definitions for IsAuthError()
@@ -239,7 +298,7 @@ func (e authError) AuthError() bool {
 }
 
 func NewAuthError(err error) error {
-	return authError{newWrappedError(err, "Authentication required")}
+	return authError{newWrappedError(err, tr.Tr.Get("Authentication required"))}
 }
 
 // Definitions for IsSmudgeError()
@@ -253,7 +312,7 @@ func (e smudgeError) SmudgeError() bool {
 }
 
 func NewSmudgeError(err error, oid, filename string) error {
-	e := smudgeError{newWrappedError(err, "Smudge error")}
+	e := smudgeError{newWrappedError(err, tr.Tr.Get("Smudge error"))}
 	SetContext(e, "OID", oid)
 	SetContext(e, "FileName", filename)
 	return e
@@ -270,7 +329,7 @@ func (e cleanPointerError) CleanPointerError() bool {
 }
 
 func NewCleanPointerError(pointer interface{}, bytes []byte) error {
-	err := New("pointer error")
+	err := New(tr.Tr.Get("pointer error"))
 	e := cleanPointerError{newWrappedError(err, "clean")}
 	SetContext(e, "pointer", pointer)
 	SetContext(e, "bytes", bytes)
@@ -288,7 +347,31 @@ func (e notAPointerError) NotAPointerError() bool {
 }
 
 func NewNotAPointerError(err error) error {
-	return notAPointerError{newWrappedError(err, "Pointer file error")}
+	return notAPointerError{newWrappedError(err, tr.Tr.Get("Pointer file error"))}
+}
+
+// Definitions for IsPointerScanError()
+
+type PointerScanError struct {
+	treeishOid string
+	path       string
+	*wrappedError
+}
+
+func (e PointerScanError) PointerScanError() bool {
+	return true
+}
+
+func (e PointerScanError) OID() string {
+	return e.treeishOid
+}
+
+func (e PointerScanError) Path() string {
+	return e.path
+}
+
+func NewPointerScanError(err error, treeishOid, path string) error {
+	return PointerScanError{treeishOid, path, newWrappedError(err, tr.Tr.Get("Pointer error"))}
 }
 
 type badPointerKeyError struct {
@@ -303,8 +386,8 @@ func (e badPointerKeyError) BadPointerKeyError() bool {
 }
 
 func NewBadPointerKeyError(expected, actual string) error {
-	err := Errorf("Expected key %s, got %s", expected, actual)
-	return badPointerKeyError{expected, actual, newWrappedError(err, "pointer parsing")}
+	err := Errorf(tr.Tr.Get("Expected key %s, got %s", expected, actual))
+	return badPointerKeyError{expected, actual, newWrappedError(err, tr.Tr.Get("pointer parsing"))}
 }
 
 // Definitions for IsDownloadDeclinedError()
@@ -321,6 +404,52 @@ func NewDownloadDeclinedError(err error, msg string) error {
 	return downloadDeclinedError{newWrappedError(err, msg)}
 }
 
+// Definitions for IsRetriableLaterError()
+
+type retriableLaterError struct {
+	*wrappedError
+	timeAvailable time.Time
+}
+
+func NewRetriableLaterError(err error, header string) error {
+	secs, err := strconv.Atoi(header)
+	if err == nil {
+		return retriableLaterError{
+			wrappedError:  newWrappedError(err, ""),
+			timeAvailable: time.Now().Add(time.Duration(secs) * time.Second),
+		}
+	}
+
+	time, err := time.Parse(time.RFC1123, header)
+	if err == nil {
+		return retriableLaterError{
+			wrappedError:  newWrappedError(err, ""),
+			timeAvailable: time,
+		}
+	}
+
+	// We could not return a successful error from the Retry-After header.
+	return nil
+}
+
+func (e retriableLaterError) RetriableLaterError() (time.Time, bool) {
+	return e.timeAvailable, true
+}
+
+// Definitions for IsUnprocessableEntityError()
+
+type unprocessableEntityError struct {
+	*wrappedError
+}
+
+func (e unprocessableEntityError) UnprocessableEntityError() bool {
+	return true
+}
+
+func NewUnprocessableEntityError(err error) error {
+	return unprocessableEntityError{newWrappedError(err, "")}
+}
+
 // Definitions for IsRetriableError()
 
 type retriableError struct {
@@ -335,6 +464,20 @@ func NewRetriableError(err error) error {
 	return retriableError{newWrappedError(err, "")}
 }
 
+// Definitions for IsProtocolError()
+
+type protocolError struct {
+	*wrappedError
+}
+
+func (e protocolError) ProtocolError() bool {
+	return true
+}
+
+func NewProtocolError(message string, err error) error {
+	return protocolError{newWrappedError(err, message)}
+}
+
 func parentOf(err error) error {
 	type causer interface {
 		Cause() error
@@ -348,3 +491,14 @@ func parentOf(err error) error {
 
 	return nil
 }
+
+func ExitStatus(err error) int {
+	var eerr *exec.ExitError
+	if goerrors.As(err, &eerr) {
+		ws, ok := eerr.ProcessState.Sys().(syscall.WaitStatus)
+		if ok {
+			return ws.ExitStatus()
+		}
+	}
+	return -1
+}
