@@ -14,18 +14,17 @@ import (
 	"io/ioutil"
 	"net/url"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"regexp"
 	"strconv"
 	"strings"
 	"sync"
-	"syscall"
 	"time"
 
 	lfserrors "github.com/git-lfs/git-lfs/v3/errors"
 	"github.com/git-lfs/git-lfs/v3/subprocess"
 	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/git-lfs/gitobj/v2"
 	"github.com/rubyist/tracerx"
 )
@@ -113,7 +112,7 @@ type Ref struct {
 // for a remote branch called 'my-feature' on remote 'origin', this function
 // will return:
 //
-//   refs/remotes/origin/my-feature
+//	refs/remotes/origin/my-feature
 func (r *Ref) Refspec() string {
 	if r == nil {
 		return ""
@@ -149,17 +148,20 @@ func IsZeroObjectID(s string) bool {
 	return false
 }
 
-func EmptyTree() string {
+func EmptyTree() (string, error) {
 	emptyTreeMutex.Lock()
 	defer emptyTreeMutex.Unlock()
 
 	if len(emptyTree) == 0 {
-		cmd := gitNoLFS("hash-object", "-t", "tree", "/dev/null")
+		cmd, err := gitNoLFS("hash-object", "-t", "tree", "/dev/null")
+		if err != nil {
+			return "", errors.New(tr.Tr.Get("failed to find `git hash-object`: %v", err))
+		}
 		cmd.Stdin = nil
 		out, _ := cmd.Output()
 		emptyTree = strings.TrimSpace(string(out))
 	}
-	return emptyTree
+	return emptyTree, nil
 }
 
 // Some top level information about a commit (only first line of message)
@@ -199,7 +201,7 @@ func gitConfigNoLFS(args ...string) []string {
 }
 
 // Invoke Git with disabled LFS filters
-func gitNoLFS(args ...string) *subprocess.Cmd {
+func gitNoLFS(args ...string) (*subprocess.Cmd, error) {
 	return subprocess.ExecCommand("git", gitConfigNoLFS(args...)...)
 }
 
@@ -212,7 +214,7 @@ func gitNoLFSBuffered(args ...string) (*subprocess.BufferedCmd, error) {
 }
 
 // Invoke Git with enabled LFS filters
-func git(args ...string) *subprocess.Cmd {
+func git(args ...string) (*subprocess.Cmd, error) {
 	return subprocess.ExecCommand("git", args...)
 }
 
@@ -232,7 +234,7 @@ func DiffIndex(ref string, cached bool, refresh bool) (*bufio.Scanner, error) {
 	if refresh {
 		_, err := gitSimple("update-index", "-q", "--refresh")
 		if err != nil {
-			return nil, lfserrors.Wrap(err, "Failed to run git update-index")
+			return nil, lfserrors.Wrap(err, tr.Tr.Get("Failed to run `git update-index`"))
 		}
 	}
 
@@ -254,11 +256,14 @@ func DiffIndex(ref string, cached bool, refresh bool) (*bufio.Scanner, error) {
 }
 
 func HashObject(r io.Reader) (string, error) {
-	cmd := gitNoLFS("hash-object", "--stdin")
+	cmd, err := gitNoLFS("hash-object", "--stdin")
+	if err != nil {
+		return "", errors.New(tr.Tr.Get("failed to find `git hash-object`: %v", err))
+	}
 	cmd.Stdin = r
 	out, err := cmd.Output()
 	if err != nil {
-		return "", fmt.Errorf("error building Git blob OID: %s", err)
+		return "", errors.New(tr.Tr.Get("error building Git blob OID: %s", err))
 	}
 
 	return string(bytes.TrimSpace(out)), nil
@@ -271,7 +276,7 @@ func Log(args ...string) (*subprocess.BufferedCmd, error) {
 
 func LsRemote(remote, remoteRef string) (string, error) {
 	if remote == "" {
-		return "", errors.New("remote required")
+		return "", errors.New(tr.Tr.Get("remote required"))
 	}
 	if remoteRef == "" {
 		return gitNoLFSSimple("ls-remote", remote)
@@ -294,10 +299,10 @@ func LsTree(ref string) (*subprocess.BufferedCmd, error) {
 func ResolveRef(ref string) (*Ref, error) {
 	outp, err := gitNoLFSSimple("rev-parse", ref, "--symbolic-full-name", ref)
 	if err != nil {
-		return nil, fmt.Errorf("Git can't resolve ref: %q", ref)
+		return nil, errors.New(tr.Tr.Get("Git can't resolve ref: %q", ref))
 	}
 	if outp == "" {
-		return nil, fmt.Errorf("Git can't resolve ref: %q", ref)
+		return nil, errors.New(tr.Tr.Get("Git can't resolve ref: %q", ref))
 	}
 
 	lines := strings.Split(outp, "\n")
@@ -351,12 +356,12 @@ func (c *Configuration) RemoteRefNameForCurrentBranch() (string, error) {
 	}
 
 	if ref.Type == RefTypeHEAD || ref.Type == RefTypeOther {
-		return "", errors.New("not on a branch")
+		return "", errors.New(tr.Tr.Get("not on a branch"))
 	}
 
 	remote := c.RemoteForBranch(ref.Name)
 	if remote == "" {
-		return "", fmt.Errorf("remote not found for branch %q", ref.Name)
+		return "", errors.New(tr.Tr.Get("remote not found for branch %q", ref.Name))
 	}
 
 	remotebranch := c.RemoteBranchForLocalBranch(ref.Name)
@@ -382,11 +387,14 @@ func (c *Configuration) RemoteBranchForLocalBranch(localBranch string) string {
 }
 
 func RemoteList() ([]string, error) {
-	cmd := gitNoLFS("remote")
+	cmd, err := gitNoLFS("remote")
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git remote`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git remote: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git remote`: %v", err))
 	}
 	cmd.Start()
 	defer cmd.Wait()
@@ -402,11 +410,14 @@ func RemoteList() ([]string, error) {
 }
 
 func RemoteURLs(push bool) (map[string][]string, error) {
-	cmd := gitNoLFS("remote", "-v")
+	cmd, err := gitNoLFS("remote", "-v")
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git remote -v`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git remote -v: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git remote -v`: %v", err))
 	}
 	cmd.Start()
 	defer cmd.Wait()
@@ -452,11 +463,14 @@ func MapRemoteURL(url string, push bool) (string, bool) {
 // Refs returns all of the local and remote branches and tags for the current
 // repository. Other refs (HEAD, refs/stash, git notes) are ignored.
 func LocalRefs() ([]*Ref, error) {
-	cmd := gitNoLFS("show-ref")
+	cmd, err := gitNoLFS("show-ref")
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git show-ref`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git show-ref: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git show-ref`: %v", err))
 	}
 
 	var refs []*Ref
@@ -470,7 +484,7 @@ func LocalRefs() ([]*Ref, error) {
 		line := strings.TrimSpace(scanner.Text())
 		parts := strings.SplitN(line, " ", 2)
 		if len(parts) != 2 || !HasValidObjectIDLength(parts[0]) || len(parts[1]) < 1 {
-			tracerx.Printf("Invalid line from git show-ref: %q", line)
+			tracerx.Printf("Invalid line from `git show-ref`: %q", line)
 			continue
 		}
 
@@ -501,7 +515,10 @@ func UpdateRefIn(wd string, ref *Ref, to []byte, reason string) error {
 		args = append(args, "-m", reason)
 	}
 
-	cmd := gitNoLFS(args...)
+	cmd, err := gitNoLFS(args...)
+	if err != nil {
+		return errors.New(tr.Tr.Get("failed to find `git update-ref`: %v", err))
+	}
 	cmd.Dir = wd
 
 	return cmd.Run()
@@ -524,7 +541,7 @@ func ValidateRemote(remote string) error {
 		return nil
 	}
 
-	return fmt.Errorf("invalid remote name: %q", remote)
+	return errors.New(tr.Tr.Get("invalid remote name: %q", remote))
 }
 
 // ValidateRemoteURL checks that a string is a valid Git remote URL
@@ -539,7 +556,7 @@ func ValidateRemoteURL(remote string) error {
 		if strings.Contains(remote, ":") {
 			return nil
 		} else {
-			return fmt.Errorf("invalid remote name: %q", remote)
+			return errors.New(tr.Tr.Get("invalid remote name: %q", remote))
 		}
 	}
 
@@ -547,7 +564,7 @@ func ValidateRemoteURL(remote string) error {
 	case "ssh", "http", "https", "git", "file":
 		return nil
 	default:
-		return fmt.Errorf("invalid remote url protocol %q in %q", u.Scheme, remote)
+		return errors.New(tr.Tr.Get("invalid remote URL protocol %q in %q", u.Scheme, remote))
 	}
 }
 
@@ -579,7 +596,7 @@ func RewriteLocalPathAsURL(path string) string {
 	return fmt.Sprintf("file://%s%s", slash, filepath.ToSlash(path))
 }
 
-func UpdateIndexFromStdin() *subprocess.Cmd {
+func UpdateIndexFromStdin() (*subprocess.Cmd, error) {
 	return git("update-index", "-q", "--refresh", "--stdin")
 }
 
@@ -589,13 +606,16 @@ func UpdateIndexFromStdin() *subprocess.Cmd {
 // includeRemoteBranches: true to include refs on remote branches
 // onlyRemote: set to non-blank to only include remote branches on a single remote
 func RecentBranches(since time.Time, includeRemoteBranches bool, onlyRemote string) ([]*Ref, error) {
-	cmd := gitNoLFS("for-each-ref",
+	cmd, err := gitNoLFS("for-each-ref",
 		`--sort=-committerdate`,
 		`--format=%(refname) %(objectname) %(committerdate:iso)`,
 		"refs")
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git for-each-ref`: %v", err))
+	}
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git for-each-ref: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git for-each-ref`: %v", err))
 	}
 	cmd.Start()
 	defer cmd.Wait()
@@ -688,12 +708,15 @@ func FormatGitDate(tm time.Time) string {
 
 // Get summary information about a commit
 func GetCommitSummary(commit string) (*CommitSummary, error) {
-	cmd := gitNoLFS("show", "-s",
+	cmd, err := gitNoLFS("show", "-s",
 		`--format=%H|%h|%P|%ai|%ci|%ae|%an|%ce|%cn|%s`, commit)
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git show`: %v", err))
+	}
 
 	out, err := cmd.CombinedOutput()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git show: %v %v", err, string(out))
+		return nil, errors.New(tr.Tr.Get("failed to call `git show`: %v %v", err, string(out)))
 	}
 
 	// At most 10 substrings so subject line is not split on anything
@@ -717,13 +740,16 @@ func GetCommitSummary(commit string) (*CommitSummary, error) {
 		}
 		return ret, nil
 	} else {
-		msg := fmt.Sprintf("Unexpected output from git show: %v", string(out))
+		msg := tr.Tr.Get("Unexpected output from `git show`: %v", string(out))
 		return nil, errors.New(msg)
 	}
 }
 
 func GitAndRootDirs() (string, string, error) {
-	cmd := gitNoLFS("rev-parse", "--git-dir", "--show-toplevel")
+	cmd, err := gitNoLFS("rev-parse", "--git-dir", "--show-toplevel")
+	if err != nil {
+		return "", "", errors.New(tr.Tr.Get("failed to find `git rev-parse --git-dir --show-toplevel`: %v", err))
+	}
 	buf := &bytes.Buffer{}
 	cmd.Stderr = buf
 
@@ -733,27 +759,23 @@ func GitAndRootDirs() (string, string, error) {
 		// If we got a fatal error, it's possible we're on a newer
 		// (2.24+) Git and we're not in a worktree, so fall back to just
 		// looking up the repo directory.
-		if e, ok := err.(*exec.ExitError); ok {
-			var ws syscall.WaitStatus
-			ws, ok = e.ProcessState.Sys().(syscall.WaitStatus)
-			if ok && ws.ExitStatus() == 128 {
-				absGitDir, err := GitDir()
-				return absGitDir, "", err
-			}
+		if lfserrors.ExitStatus(err) == 128 {
+			absGitDir, err := GitDir()
+			return absGitDir, "", err
 		}
-		return "", "", fmt.Errorf("failed to call git rev-parse --git-dir --show-toplevel: %q", buf.String())
+		return "", "", errors.New(tr.Tr.Get("failed to call `git rev-parse --git-dir --show-toplevel`: %q", buf.String()))
 	}
 
 	paths := strings.Split(output, "\n")
 	pathLen := len(paths)
 
 	if pathLen == 0 {
-		return "", "", fmt.Errorf("bad git rev-parse output: %q", output)
+		return "", "", errors.New(tr.Tr.Get("bad `git rev-parse` output: %q", output))
 	}
 
 	absGitDir, err := tools.CanonicalizePath(paths[0], false)
 	if err != nil {
-		return "", "", fmt.Errorf("error converting %q to absolute: %s", paths[0], err)
+		return "", "", errors.New(tr.Tr.Get("error converting %q to absolute: %s", paths[0], err))
 	}
 
 	if pathLen == 1 || len(paths[1]) == 0 {
@@ -765,10 +787,13 @@ func GitAndRootDirs() (string, string, error) {
 }
 
 func RootDir() (string, error) {
-	cmd := gitNoLFS("rev-parse", "--show-toplevel")
+	cmd, err := gitNoLFS("rev-parse", "--show-toplevel")
+	if err != nil {
+		return "", errors.New(tr.Tr.Get("failed to find `git rev-parse --show-toplevel`: %v", err))
+	}
 	out, err := cmd.Output()
 	if err != nil {
-		return "", fmt.Errorf("failed to call git rev-parse --show-toplevel: %v %v", err, string(out))
+		return "", errors.New(tr.Tr.Get("failed to call `git rev-parse --show-toplevel`: %v %v", err, string(out)))
 	}
 
 	path := strings.TrimSpace(string(out))
@@ -780,13 +805,20 @@ func RootDir() (string, error) {
 }
 
 func GitDir() (string, error) {
-	cmd := gitNoLFS("rev-parse", "--git-dir")
+	cmd, err := gitNoLFS("rev-parse", "--git-dir")
+	if err != nil {
+		// The %w format specifier is unique to fmt.Errorf(), so we
+		// do not pass it to tr.Tr.Get().
+		return "", fmt.Errorf("%s: %w", tr.Tr.Get("failed to find `git rev-parse --git-dir`"), err)
+	}
 	buf := &bytes.Buffer{}
 	cmd.Stderr = buf
 	out, err := cmd.Output()
 
 	if err != nil {
-		return "", fmt.Errorf("failed to call git rev-parse --git-dir: %v %v: %v", err, string(out), buf.String())
+		// The %w format specifier is unique to fmt.Errorf(), so we
+		// do not pass it to tr.Tr.Get().
+		return "", fmt.Errorf("%s: %w %v: %v", tr.Tr.Get("failed to call `git rev-parse --git-dir`"), err, string(out), buf.String())
 	}
 	path := strings.TrimSpace(string(out))
 	return tools.CanonicalizePath(path, false)
@@ -800,12 +832,15 @@ func GitCommonDir() (string, error) {
 		return GitDir()
 	}
 
-	cmd := gitNoLFS("rev-parse", "--git-common-dir")
+	cmd, err := gitNoLFS("rev-parse", "--git-common-dir")
+	if err != nil {
+		return "", errors.New(tr.Tr.Get("failed to find `git rev-parse --git-common-dir`: %v", err))
+	}
 	out, err := cmd.Output()
 	buf := &bytes.Buffer{}
 	cmd.Stderr = buf
 	if err != nil {
-		return "", fmt.Errorf("failed to call git rev-parse --git-common-dir: %v %v: %v", err, string(out), buf.String())
+		return "", errors.New(tr.Tr.Get("failed to call `git rev-parse --git-common-dir`: %v %v: %v", err, string(out), buf.String()))
 	}
 	path := strings.TrimSpace(string(out))
 	path, err = tools.TranslateCygwinPath(path)
@@ -852,7 +887,7 @@ func GetAllWorkTreeHEADs(storageDir string) ([]*Ref, error) {
 
 	// This has only established the separate worktrees, not the original checkout
 	// If the storageDir contains a HEAD file then there is a main checkout
-	// as well; this mus tbe resolveable whether you're in the main checkout or
+	// as well; this must be resolveable whether you're in the main checkout or
 	// a worktree
 	headfile := filepath.Join(storageDir, "HEAD")
 	ref, err := parseRefFile(headfile)
@@ -1058,21 +1093,24 @@ func CloneWithoutFilters(flags CloneFlags, args []string) error {
 
 	// Now args
 	cmdargs = append(cmdargs, args...)
-	cmd := gitNoLFS(cmdargs...)
+	cmd, err := gitNoLFS(cmdargs...)
+	if err != nil {
+		return errors.New(tr.Tr.Get("failed to find `git clone`: %v", err))
+	}
 
 	// Assign all streams direct
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
 	cmd.Stdin = os.Stdin
 
-	err := cmd.Start()
+	err = cmd.Start()
 	if err != nil {
-		return fmt.Errorf("failed to start git clone: %v", err)
+		return errors.New(tr.Tr.Get("failed to start `git clone`: %v", err))
 	}
 
 	err = cmd.Wait()
 	if err != nil {
-		return fmt.Errorf("git clone failed: %v", err)
+		return errors.New(tr.Tr.Get("`git clone` failed: %v", err))
 	}
 
 	return nil
@@ -1105,11 +1143,14 @@ func Checkout(treeish string, paths []string, force bool) error {
 // currently cached locally. No remote request is made to verify them.
 func CachedRemoteRefs(remoteName string) ([]*Ref, error) {
 	var ret []*Ref
-	cmd := gitNoLFS("show-ref")
+	cmd, err := gitNoLFS("show-ref")
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git show-ref`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git show-ref: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git show-ref`: %v", err))
 	}
 	cmd.Start()
 	scanner := bufio.NewScanner(outp)
@@ -1160,11 +1201,14 @@ func Fetch(remotes ...string) error {
 // accessing the remote via git ls-remote.
 func RemoteRefs(remoteName string) ([]*Ref, error) {
 	var ret []*Ref
-	cmd := gitNoLFS("ls-remote", "--heads", "--tags", "-q", remoteName)
+	cmd, err := gitNoLFS("ls-remote", "--heads", "--tags", "-q", remoteName)
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git ls-remote`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git ls-remote: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git ls-remote`: %v", err))
 	}
 	cmd.Start()
 	scanner := bufio.NewScanner(outp)
@@ -1219,13 +1263,16 @@ func AllRefs() ([]*Ref, error) {
 // the given working directory "wd", or an error if those references could not
 // be loaded.
 func AllRefsIn(wd string) ([]*Ref, error) {
-	cmd := gitNoLFS(
+	cmd, err := gitNoLFS(
 		"for-each-ref", "--format=%(objectname)%00%(refname)")
+	if err != nil {
+		return nil, lfserrors.Wrap(err, tr.Tr.Get("failed to find `git for-each-ref`: %v", err))
+	}
 	cmd.Dir = wd
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, lfserrors.Wrap(err, "cannot open pipe")
+		return nil, lfserrors.Wrap(err, tr.Tr.Get("cannot open pipe"))
 	}
 	cmd.Start()
 
@@ -1235,8 +1282,8 @@ func AllRefsIn(wd string) ([]*Ref, error) {
 	for scanner.Scan() {
 		parts := strings.SplitN(scanner.Text(), "\x00", 2)
 		if len(parts) != 2 {
-			return nil, lfserrors.Errorf(
-				"git: invalid for-each-ref line: %q", scanner.Text())
+			return nil, lfserrors.New(tr.Tr.Get(
+				"invalid `git for-each-ref` line: %q", scanner.Text()))
 		}
 
 		sha := parts[0]
@@ -1265,16 +1312,19 @@ func GetTrackedFiles(pattern string) ([]string, error) {
 	rootWildcard := len(safePattern) < len(pattern) && strings.ContainsRune(safePattern, '*')
 
 	var ret []string
-	cmd := gitNoLFS(
+	cmd, err := gitNoLFS(
 		"-c", "core.quotepath=false", // handle special chars in filenames
 		"ls-files",
 		"--cached", // include things which are staged but not committed right now
 		"--",       // no ambiguous patterns
 		safePattern)
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git ls-files`: %v", err))
+	}
 
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git ls-files: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git ls-files`: %v", err))
 	}
 	cmd.Start()
 	scanner := bufio.NewScanner(outp)
@@ -1324,20 +1374,23 @@ func GetFilesChanged(from, to string) ([]string, error) {
 	}
 	args = append(args, "--") // no ambiguous patterns
 
-	cmd := gitNoLFS(args...)
+	cmd, err := gitNoLFS(args...)
+	if err != nil {
+		return nil, errors.New(tr.Tr.Get("failed to find `git diff-tree`: %v", err))
+	}
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, fmt.Errorf("failed to call git diff: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to call `git diff-tree`: %v", err))
 	}
 	if err := cmd.Start(); err != nil {
-		return nil, fmt.Errorf("failed to start git diff: %v", err)
+		return nil, errors.New(tr.Tr.Get("failed to start `git diff-tree`: %v", err))
 	}
 	scanner := bufio.NewScanner(outp)
 	for scanner.Scan() {
 		files = append(files, strings.TrimSpace(scanner.Text()))
 	}
 	if err := cmd.Wait(); err != nil {
-		return nil, fmt.Errorf("git diff failed: %v", err)
+		return nil, errors.New(tr.Tr.Get("`git diff-tree` failed: %v", err))
 	}
 
 	return files, err
@@ -1355,13 +1408,16 @@ func IsFileModified(filepath string) (bool, error) {
 		"--", // separator in case filename ambiguous
 		filepath,
 	}
-	cmd := git(args...)
+	cmd, err := git(args...)
+	if err != nil {
+		return false, lfserrors.Wrap(err, tr.Tr.Get("failed to find `git status`"))
+	}
 	outp, err := cmd.StdoutPipe()
 	if err != nil {
-		return false, lfserrors.Wrap(err, "Failed to call git status")
+		return false, lfserrors.Wrap(err, tr.Tr.Get("Failed to call `git status`"))
 	}
 	if err := cmd.Start(); err != nil {
-		return false, lfserrors.Wrap(err, "Failed to start git status")
+		return false, lfserrors.Wrap(err, tr.Tr.Get("Failed to start `git status`"))
 	}
 	matched := false
 	for scanner := bufio.NewScanner(outp); scanner.Scan(); {
@@ -1378,7 +1434,7 @@ func IsFileModified(filepath string) (bool, error) {
 		}
 	}
 	if err := cmd.Wait(); err != nil {
-		return false, lfserrors.Wrap(err, "Git status failed")
+		return false, lfserrors.Wrap(err, tr.Tr.Get("`git status` failed"))
 	}
 
 	return matched, nil
@@ -1404,6 +1460,10 @@ func IsWorkingCopyDirty() (bool, error) {
 
 func ObjectDatabase(osEnv, gitEnv Environment, gitdir, tempdir string) (*gitobj.ObjectDatabase, error) {
 	var options []gitobj.Option
+	objdir, ok := osEnv.Get("GIT_OBJECT_DIRECTORY")
+	if !ok {
+		objdir = filepath.Join(gitdir, "objects")
+	}
 	alternates, _ := osEnv.Get("GIT_ALTERNATE_OBJECT_DIRECTORIES")
 	if alternates != "" {
 		options = append(options, gitobj.Alternates(alternates))
@@ -1412,12 +1472,80 @@ func ObjectDatabase(osEnv, gitEnv Environment, gitdir, tempdir string) (*gitobj.
 	if hashAlgo != "" {
 		options = append(options, gitobj.ObjectFormat(gitobj.ObjectFormatAlgorithm(hashAlgo)))
 	}
-	odb, err := gitobj.FromFilesystem(filepath.Join(gitdir, "objects"), tempdir, options...)
+	odb, err := gitobj.FromFilesystem(objdir, tempdir, options...)
 	if err != nil {
 		return nil, err
 	}
 	if odb.Hasher() == nil {
-		return nil, fmt.Errorf("unsupported repository hash algorithm %q", hashAlgo)
+		return nil, errors.New(tr.Tr.Get("unsupported repository hash algorithm %q", hashAlgo))
 	}
 	return odb, nil
 }
+
+func remotesForTreeish(treeish string) []string {
+	var outp string
+	var err error
+	if treeish == "" {
+		//Treeish is empty for sparse checkout
+		tracerx.Printf("git: treeish: not provided")
+		outp, err = gitNoLFSSimple("branch", "-r", "--contains", "HEAD")
+	} else {
+		tracerx.Printf("git: treeish: %q", treeish)
+		outp, err = gitNoLFSSimple("branch", "-r", "--contains", treeish)
+	}
+	if err != nil || outp == "" {
+		tracerx.Printf("git: symbolic name: can't resolve symbolic name for ref: %q", treeish)
+		return []string{}
+	}
+	return strings.Split(outp, "\n")
+}
+
+// remoteForRef will try to determine the remote from the ref name.
+// This will return an empty string if any of the remote names have a slash
+// because slashes introduce ambiguity. Consider two refs:
+//
+// 1. upstream/main
+// 2. upstream/test/main
+//
+// Is the remote "upstream" or "upstream/test"? It could be either, or both.
+// We could use git for-each-ref with %(upstream:remotename) if there were a tracking branch,
+// but this is not guaranteed to exist either.
+func remoteForRef(refname string) string {
+	tracerx.Printf("git: working ref: %s", refname)
+	remotes, err := RemoteList()
+	if err != nil {
+		return ""
+	}
+	parts := strings.Split(refname, "/")
+	if len(parts) < 2 {
+		return ""
+	}
+	for _, name := range remotes {
+		if strings.Contains(name, "/") {
+			tracerx.Printf("git: ref remote: cannot determine remote for ref %s since remote %s contains a slash", refname, name)
+			return ""
+		}
+	}
+	remote := parts[0]
+	tracerx.Printf("git: working remote %s", remote)
+	return remote
+}
+
+func getValidRemote(refs []string) string {
+	for _, ref := range refs {
+		if ref != "" {
+			return ref
+		}
+	}
+	return ""
+}
+
+// FirstRemoteForTreeish returns the first remote found which contains the treeish.
+func FirstRemoteForTreeish(treeish string) string {
+	name := getValidRemote(remotesForTreeish(treeish))
+	if name == "" {
+		tracerx.Printf("git: remote treeish: no valid remote refs parsed for %q", treeish)
+		return ""
+	}
+	return remoteForRef(name)
+}
