@@ -12,6 +12,7 @@ import (
 	"github.com/git-lfs/git-lfs/v3/filepathfilter"
 	"github.com/git-lfs/git-lfs/v3/git"
 	"github.com/git-lfs/git-lfs/v3/tasklog"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/git-lfs/gitobj/v2"
 )
 
@@ -77,7 +78,7 @@ type RewriteOptions struct {
 	TreeCallbackFn TreeCallbackFn
 }
 
-// blobFn returns a useable BlobRewriteFn, either the one that was given in the
+// blobFn returns a usable BlobRewriteFn, either the one that was given in the
 // *RewriteOptions, or a noopBlobFn.
 func (r *RewriteOptions) blobFn() BlobRewriteFn {
 	if r.BlobFn == nil {
@@ -86,7 +87,7 @@ func (r *RewriteOptions) blobFn() BlobRewriteFn {
 	return r.BlobFn
 }
 
-// treePreFn returns a useable TreePreCallbackFn, either the one that was given
+// treePreFn returns a usable TreePreCallbackFn, either the one that was given
 // in the *RewriteOptions, or a noopTreePreFn.
 func (r *RewriteOptions) treePreFn() TreePreCallbackFn {
 	if r.TreePreCallbackFn == nil {
@@ -95,7 +96,7 @@ func (r *RewriteOptions) treePreFn() TreePreCallbackFn {
 	return r.TreePreCallbackFn
 }
 
-// treeFn returns a useable TreeRewriteFn, either the one that was given in the
+// treeFn returns a usable TreeRewriteFn, either the one that was given in the
 // *RewriteOptions, or a noopTreeFn.
 func (r *RewriteOptions) treeFn() TreeCallbackFn {
 	if r.TreeCallbackFn == nil {
@@ -138,7 +139,6 @@ type TreePreCallbackFn func(path string, t *gitobj.Tree) error
 // written to the object database instead of one generated from calling BlobFn
 // on all of the tree entries.
 //
-//
 // TreeCallbackFn can be nil, and will therefore exhibit behavior equivalent to
 // only calling the BlobFn on existing tree entries.
 //
@@ -201,8 +201,9 @@ func NewRewriter(db *gitobj.ObjectDatabase, opts ...rewriterOption) *Rewriter {
 	return rewriter
 }
 
-// Rewrite rewrites the range of commits given by *RewriteOptions.{Left,Right}
-// using the BlobRewriteFn to rewrite the individual blobs.
+// Rewrite rewrites the range of commits given by
+// *RewriteOptions.{Include,Exclude} using the BlobRewriteFn to rewrite
+// the individual blobs.
 func (r *Rewriter) Rewrite(opt *RewriteOptions) ([]byte, error) {
 	// First, obtain a list of commits to rewrite.
 	commits, err := r.commitsToMigrate(opt)
@@ -212,9 +213,9 @@ func (r *Rewriter) Rewrite(opt *RewriteOptions) ([]byte, error) {
 
 	var perc *tasklog.PercentageTask
 	if opt.UpdateRefs {
-		perc = r.l.Percentage("migrate: Rewriting commits", uint64(len(commits)))
+		perc = r.l.Percentage(fmt.Sprintf("migrate: %s", tr.Tr.Get("Rewriting commits")), uint64(len(commits)))
 	} else {
-		perc = r.l.Percentage("migrate: Examining commits", uint64(len(commits)))
+		perc = r.l.Percentage(fmt.Sprintf("migrate: %s", tr.Tr.Get("Examining commits")), uint64(len(commits)))
 	}
 
 	var vPerc *tasklog.PercentageTask
@@ -226,7 +227,7 @@ func (r *Rewriter) Rewrite(opt *RewriteOptions) ([]byte, error) {
 	if len(opt.ObjectMapFilePath) > 0 {
 		objectMapFile, err = os.OpenFile(opt.ObjectMapFilePath, os.O_RDWR|os.O_CREATE|os.O_EXCL, 0666)
 		if err != nil {
-			return nil, fmt.Errorf("could not create object map file: %v", err)
+			return nil, errors.New(tr.Tr.Get("could not create object map file: %v", err))
 		}
 		defer objectMapFile.Close()
 	}
@@ -317,7 +318,7 @@ func (r *Rewriter) Rewrite(opt *RewriteOptions) ([]byte, error) {
 	if opt.UpdateRefs {
 		refs, err := r.refsToMigrate()
 		if err != nil {
-			return nil, errors.Wrap(err, "could not find refs to update")
+			return nil, errors.Wrap(err, tr.Tr.Get("could not find refs to update"))
 		}
 
 		root, _ := r.db.Root()
@@ -332,7 +333,7 @@ func (r *Rewriter) Rewrite(opt *RewriteOptions) ([]byte, error) {
 		}
 
 		if err := updater.UpdateRefs(); err != nil {
-			return nil, errors.Wrap(err, "could not update refs")
+			return nil, errors.Wrap(err, tr.Tr.Get("could not update refs"))
 		}
 	}
 
@@ -383,7 +384,7 @@ func (r *Rewriter) rewriteTree(commitOID []byte, treeOID []byte, path string,
 			continue
 		}
 
-		if cached := r.uncacheEntry(entry); cached != nil {
+		if cached := r.uncacheEntry(fullpath, entry); cached != nil {
 			entries = append(entries, copyEntryMode(cached,
 				entry.Filemode))
 			continue
@@ -404,7 +405,7 @@ func (r *Rewriter) rewriteTree(commitOID []byte, treeOID []byte, path string,
 			return nil, err
 		}
 
-		entries = append(entries, r.cacheEntry(entry, &gitobj.TreeEntry{
+		entries = append(entries, r.cacheEntry(fullpath, entry, &gitobj.TreeEntry{
 			Filemode: entry.Filemode,
 			Name:     entry.Name,
 			Oid:      oid,
@@ -451,7 +452,7 @@ func (r *Rewriter) allows(typ gitobj.ObjectType, abs string) bool {
 	case gitobj.CommitObjectType, gitobj.TreeObjectType:
 		return true
 	default:
-		panic(fmt.Sprintf("git/githistory: unknown entry type: %s", typ))
+		panic(fmt.Sprintf("git/githistory: %s", tr.Tr.Get("unknown entry type: %s", typ)))
 	}
 }
 
@@ -488,7 +489,7 @@ func (r *Rewriter) rewriteBlob(commitOID, from []byte, path string, fn BlobRewri
 		}
 
 		if perc != nil {
-			perc.Entry(fmt.Sprintf("migrate: commit %s: %s", hex.EncodeToString(commitOID), path))
+			perc.Entry(fmt.Sprintf("migrate: %s", tr.Tr.Get("commit %s: %s", hex.EncodeToString(commitOID), path)))
 		}
 
 		return sha, nil
@@ -508,7 +509,7 @@ func (r *Rewriter) rewriteBlob(commitOID, from []byte, path string, fn BlobRewri
 //
 // If any error was encountered, it will be returned.
 func (r *Rewriter) commitsToMigrate(opt *RewriteOptions) ([][]byte, error) {
-	waiter := r.l.Waiter("migrate: Sorting commits")
+	waiter := r.l.Waiter(fmt.Sprintf("migrate: %s", tr.Tr.Get("Sorting commits")))
 	defer waiter.Complete()
 
 	scanner, err := git.NewRevListScanner(
@@ -590,11 +591,11 @@ func (r *Rewriter) Filter() *filepathfilter.Filter {
 
 // cacheEntry caches then given "from" entry so that it is always rewritten as
 // a *TreeEntry equivalent to "to".
-func (r *Rewriter) cacheEntry(from, to *gitobj.TreeEntry) *gitobj.TreeEntry {
+func (r *Rewriter) cacheEntry(path string, from, to *gitobj.TreeEntry) *gitobj.TreeEntry {
 	r.mu.Lock()
 	defer r.mu.Unlock()
 
-	r.entries[r.entryKey(from)] = to
+	r.entries[r.entryKey(path, from)] = to
 
 	return to
 }
@@ -602,16 +603,16 @@ func (r *Rewriter) cacheEntry(from, to *gitobj.TreeEntry) *gitobj.TreeEntry {
 // uncacheEntry returns a *TreeEntry that is cached from the given *TreeEntry
 // "from". That is to say, it returns the *TreeEntry that "from" should be
 // rewritten to, or nil if none could be found.
-func (r *Rewriter) uncacheEntry(from *gitobj.TreeEntry) *gitobj.TreeEntry {
+func (r *Rewriter) uncacheEntry(path string, from *gitobj.TreeEntry) *gitobj.TreeEntry {
 	r.mu.Lock()
 	defer r.mu.Unlock()
 
-	return r.entries[r.entryKey(from)]
+	return r.entries[r.entryKey(path, from)]
 }
 
 // entryKey returns a unique key for a given *TreeEntry "e".
-func (r *Rewriter) entryKey(e *gitobj.TreeEntry) string {
-	return fmt.Sprintf("%s:%x", e.Name, e.Oid)
+func (r *Rewriter) entryKey(path string, e *gitobj.TreeEntry) string {
+	return fmt.Sprintf("%s:%x", path, e.Oid)
 }
 
 // cacheEntry caches then given "from" commit so that it is always rewritten as
