@@ -15,21 +15,25 @@ begin_test "migrate import (default branch)"
 
   git lfs migrate import
 
-  assert_pointer "refs/heads/master" "a.md" "$md_oid" "140"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_oid" "120"
+  assert_pointer "refs/heads/main" "a.md" "$md_oid" "140"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_oid" "120"
 
   assert_local_object "$md_oid" "140"
   assert_local_object "$txt_oid" "120"
   refute_local_object "$md_feature_oid" "30"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   [ ! $(git cat-file -p "$feature:.gitattributes") ]
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
+
+  # Ensure that hooks are installed. If we find 'git-lfs' somewhere in
+  # .git/hooks/pre-push we assume that the rest went correctly, too.
+  grep -q "git-lfs" .git/hooks/pre-push
 )
 end_test
 
@@ -57,21 +61,21 @@ begin_test "migrate import (given branch)"
 
   assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "30"
   assert_pointer "refs/heads/my-feature" "a.txt" "$txt_oid" "120"
-  assert_pointer "refs/heads/master" "a.md" "$md_oid" "140"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_oid" "120"
+  assert_pointer "refs/heads/main" "a.md" "$md_oid" "140"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_oid" "120"
 
   assert_local_object "$md_oid" "140"
   assert_local_object "$md_feature_oid" "30"
   assert_local_object "$txt_oid" "120"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
 )
@@ -89,20 +93,20 @@ begin_test "migrate import (default branch with filter)"
 
   git lfs migrate import --include "*.md"
 
-  assert_pointer "refs/heads/master" "a.md" "$md_oid" "140"
+  assert_pointer "refs/heads/main" "a.md" "$md_oid" "140"
 
   assert_local_object "$md_oid" "140"
   refute_local_object "$txt_oid" "120"
   refute_local_object "$md_feature_oid" "30"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   [ ! $(git cat-file -p "$feature:.gitattributes") ]
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
 )
 end_test
 
@@ -125,14 +129,14 @@ begin_test "migrate import (given branch with filter)"
   assert_local_object "$md_feature_oid" "30"
   refute_local_object "$txt_oid" "120"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
 )
@@ -144,29 +148,29 @@ begin_test "migrate import (default branch, exclude remote refs)"
 
   setup_single_remote_branch
 
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
-  md_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  txt_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
+  md_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  txt_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
 
   git lfs migrate import
 
-  assert_pointer "refs/heads/master" "a.md" "$md_oid" "50"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_oid" "30"
+  assert_pointer "refs/heads/main" "a.md" "$md_oid" "50"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_oid" "30"
 
   assert_local_object "$md_oid" "50"
   assert_local_object "$txt_oid" "30"
   refute_local_object "$md_remote_oid" "140"
   refute_local_object "$txt_remote_oid" "120"
 
-  master="$(git rev-parse refs/heads/master)"
-  remote="$(git rev-parse refs/remotes/origin/master)"
+  main="$(git rev-parse refs/heads/main)"
+  remote="$(git rev-parse refs/remotes/origin/main)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   [ ! $(git cat-file -p "$remote:.gitattributes") ]
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
 )
 end_test
 
@@ -176,37 +180,37 @@ begin_test "migrate import (given branch, exclude remote refs)"
 
   setup_multiple_remote_branches
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
   md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
   txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
 
   git lfs migrate import my-feature
 
-  assert_pointer "refs/heads/master" "a.md" "$md_master_oid" "21"
+  assert_pointer "refs/heads/main" "a.md" "$md_main_oid" "21"
   assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "31"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_master_oid" "20"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "20"
   assert_pointer "refs/heads/my-feature" "a.txt" "$txt_feature_oid" "30"
 
   assert_local_object "$md_feature_oid" "31"
-  assert_local_object "$md_master_oid" "21"
+  assert_local_object "$md_main_oid" "21"
   assert_local_object "$txt_feature_oid" "30"
-  assert_local_object "$txt_master_oid" "20"
+  assert_local_object "$txt_main_oid" "20"
   refute_local_object "$md_remote_oid" "11"
   refute_local_object "$txt_remote_oid" "10"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
-  remote="$(git rev-parse refs/remotes/origin/master)"
+  remote="$(git rev-parse refs/remotes/origin/main)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   [ ! $(git cat-file -p "$remote:.gitattributes") ]
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
   echo "$feature_attrs" | grep -vq "*.txt filter=lfs diff=lfs merge=lfs"
 )
@@ -218,36 +222,36 @@ begin_test "migrate import (given ref, --skip-fetch)"
 
   setup_single_remote_branch
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
 
-  git tag pseudo-remote "$(git rev-parse refs/remotes/origin/master)"
-  # Remove the refs/remotes/origin/master ref, and instruct 'git lfs migrate' to
+  git tag pseudo-remote "$(git rev-parse refs/remotes/origin/main)"
+  # Remove the refs/remotes/origin/main ref, and instruct 'git lfs migrate' to
   # not fetch it.
-  git update-ref -d refs/remotes/origin/master
+  git update-ref -d refs/remotes/origin/main
 
   git lfs migrate import --skip-fetch
 
-  assert_pointer "refs/heads/master" "a.md" "$md_master_oid" "50"
+  assert_pointer "refs/heads/main" "a.md" "$md_main_oid" "50"
   assert_pointer "pseudo-remote" "a.md" "$md_remote_oid" "140"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_master_oid" "30"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "30"
   assert_pointer "pseudo-remote" "a.txt" "$txt_remote_oid" "120"
 
-  assert_local_object "$md_master_oid" "50"
-  assert_local_object "$txt_master_oid" "30"
+  assert_local_object "$md_main_oid" "50"
+  assert_local_object "$txt_main_oid" "30"
   assert_local_object "$md_remote_oid" "140"
   assert_local_object "$txt_remote_oid" "120"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   remote="$(git rev-parse pseudo-remote)"
 
-  master_attrs="$(git cat-file -p "$master:.gitattributes")"
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
   remote_attrs="$(git cat-file -p "$remote:.gitattributes")"
 
-  echo "$master_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
-  echo "$master_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
   echo "$remote_attrs" | grep -q "*.md filter=lfs diff=lfs merge=lfs"
   echo "$remote_attrs" | grep -q "*.txt filter=lfs diff=lfs merge=lfs"
 )
@@ -259,12 +263,12 @@ begin_test "migrate import (un-annotated tags)"
 
   setup_single_local_branch_with_tags
 
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
 
   git lfs migrate import --everything
 
-  assert_pointer "refs/heads/master" "a.txt" "$txt_master_oid" "2"
-  assert_local_object "$txt_master_oid" "2"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "2"
+  assert_local_object "$txt_main_oid" "2"
 
   git tag --points-at "$(git rev-parse HEAD)" | grep -q "v1.0.0"
 )
@@ -276,12 +280,12 @@ begin_test "migrate import (annotated tags)"
 
   setup_single_local_branch_with_annotated_tags
 
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
 
   git lfs migrate import --everything
 
-  assert_pointer "refs/heads/master" "a.txt" "$txt_master_oid" "2"
-  assert_local_object "$txt_master_oid" "2"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "2"
+  assert_local_object "$txt_main_oid" "2"
 
   git tag --points-at "$(git rev-parse HEAD)" | grep -q "v1.0.0"
 )
@@ -293,32 +297,32 @@ begin_test "migrate import (include/exclude ref)"
 
   setup_multiple_remote_branches
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
   md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
   txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
 
   git lfs migrate import \
     --include-ref=refs/heads/my-feature \
-    --exclude-ref=refs/heads/master
+    --exclude-ref=refs/heads/main
 
   assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "31"
   assert_pointer "refs/heads/my-feature" "a.txt" "$txt_feature_oid" "30"
 
   assert_local_object "$md_feature_oid" "31"
-  refute_local_object "$md_master_oid" "21"
+  refute_local_object "$md_main_oid" "21"
   assert_local_object "$txt_feature_oid" "30"
-  refute_local_object "$txt_master_oid" "20"
+  refute_local_object "$txt_main_oid" "20"
   refute_local_object "$md_remote_oid" "11"
   refute_local_object "$txt_remote_oid" "10"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
-  remote="$(git rev-parse refs/remotes/origin/master)"
+  remote="$(git rev-parse refs/remotes/origin/main)"
 
-  [ ! $(git cat-file -p "$master:.gitattributes") ]
+  [ ! $(git cat-file -p "$main:.gitattributes") ]
   [ ! $(git cat-file -p "$remote:.gitattributes") ]
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
@@ -333,30 +337,30 @@ begin_test "migrate import (include/exclude ref args)"
 
   setup_multiple_remote_branches
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
   md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
   txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
 
-  git lfs migrate import my-feature ^master
+  git lfs migrate import my-feature ^main
 
   assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "31"
   assert_pointer "refs/heads/my-feature" "a.txt" "$txt_feature_oid" "30"
 
   assert_local_object "$md_feature_oid" "31"
-  refute_local_object "$md_master_oid" "21"
+  refute_local_object "$md_main_oid" "21"
   assert_local_object "$txt_feature_oid" "30"
-  refute_local_object "$txt_master_oid" "20"
+  refute_local_object "$txt_main_oid" "20"
   refute_local_object "$md_remote_oid" "11"
   refute_local_object "$txt_remote_oid" "10"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
-  remote="$(git rev-parse refs/remotes/origin/master)"
+  remote="$(git rev-parse refs/remotes/origin/main)"
 
-  [ ! $(git cat-file -p "$master:.gitattributes") ]
+  [ ! $(git cat-file -p "$main:.gitattributes") ]
   [ ! $(git cat-file -p "$remote:.gitattributes") ]
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
@@ -371,32 +375,32 @@ begin_test "migrate import (include/exclude ref with filter)"
 
   setup_multiple_remote_branches
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.md")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  md_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.md")")"
   md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
-  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/master:a.txt")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  txt_remote_oid="$(calc_oid "$(git cat-file -p "refs/remotes/origin/main:a.txt")")"
   txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
 
   git lfs migrate import \
     --include="*.txt" \
     --include-ref=refs/heads/my-feature \
-    --exclude-ref=refs/heads/master
+    --exclude-ref=refs/heads/main
 
   assert_pointer "refs/heads/my-feature" "a.txt" "$txt_feature_oid" "30"
 
   refute_local_object "$md_feature_oid" "31"
-  refute_local_object "$md_master_oid" "21"
+  refute_local_object "$md_main_oid" "21"
   assert_local_object "$txt_feature_oid" "30"
-  refute_local_object "$txt_master_oid" "20"
+  refute_local_object "$txt_main_oid" "20"
   refute_local_object "$md_remote_oid" "11"
   refute_local_object "$txt_remote_oid" "10"
 
-  master="$(git rev-parse refs/heads/master)"
+  main="$(git rev-parse refs/heads/main)"
   feature="$(git rev-parse refs/heads/my-feature)"
-  remote="$(git rev-parse refs/remotes/origin/master)"
+  remote="$(git rev-parse refs/remotes/origin/main)"
 
-  [ ! $(git cat-file -p "$master:.gitattributes") ]
+  [ ! $(git cat-file -p "$main:.gitattributes") ]
   [ ! $(git cat-file -p "$remote:.gitattributes") ]
   feature_attrs="$(git cat-file -p "$feature:.gitattributes")"
 
@@ -405,26 +409,117 @@ begin_test "migrate import (include/exclude ref with filter)"
 )
 end_test
 
+begin_test "migrate import (above)"
+(
+  set -e
+  setup_single_local_branch_untracked
+
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+
+  git lfs migrate import --above 121B
+  # Ensure that 'a.md', whose size is above our 121 byte threshold
+  # was converted into a git-lfs pointer by the migration.
+  assert_local_object "$md_main_oid" "140"
+  assert_pointer "refs/heads/main" "a.md" "$md_main_oid" "140"
+  refute_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "120"
+  refute_local_object "$txt_main_oid" "120"
+
+  # The migration should have identified that *.md files are now
+  # tracked because it migrated a.md
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
+
+  echo "$main_attrs" | grep -q "/a.md filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -vq "/a.txt filter=lfs diff=lfs merge=lfs"
+  git check-attr filter -- a.txt | grep -vq lfs
+)
+end_test
+
+begin_test "migrate import (above without extension)"
+(
+  set -e
+  setup_single_local_branch_untracked "just-b"
+
+  b_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:just-b")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+
+  git lfs migrate import --above 121B
+  # Ensure that 'b', whose size is above our 121 byte threshold
+  # was converted into a git-lfs pointer by the migration.
+  assert_local_object "$b_main_oid" "140"
+  assert_pointer "refs/heads/main" "just-b" "$b_main_oid" "140"
+  refute_pointer "refs/heads/main" "a.txt" "$txt_main_oid" "120"
+  refute_local_object "$txt_main_oid" "120"
+
+  # The migration should have identified that /b is now tracked
+  # because it migrated it.
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
+
+  echo "$main_attrs" | grep -q "/just-b filter=lfs diff=lfs merge=lfs"
+  echo "$main_attrs" | grep -vq "/a.txt filter=lfs diff=lfs merge=lfs"
+  git check-attr filter -- a.txt | grep -vq lfs
+)
+end_test
+
+begin_test "migrate import (above with multiple files)"
+(
+  set -e
+  # It is important that this file sort after "a.txt".
+  setup_single_local_branch_untracked "b.txt"
+
+  a_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  b_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:b.txt")")"
+
+  git lfs migrate import --above 121B
+  # Ensure that 'a.md', whose size is above our 121 byte threshold
+  # was converted into a git-lfs pointer by the migration.
+  assert_local_object "$b_main_oid" "140"
+  assert_pointer "refs/heads/main" "b.txt" "$b_main_oid" "140"
+  refute_pointer "refs/heads/main" "a.txt" "$a_main_oid" "120"
+  refute_local_object "$a_main_oid" "120"
+
+  # The migration should have identified that *.md files are now
+  # tracked because it migrated a.md
+  main_attrs="$(git cat-file -p "$main:.gitattributes")"
+
+  echo "$main_attrs" | grep -q "/b.txt filter=lfs diff=lfs merge=lfs"
+  git check-attr filter -- a.txt | grep -vq lfs
+)
+end_test
+
+begin_test "migrate import (above with include or exclude)"
+(
+  set -e
+  setup_single_local_branch_untracked
+
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+
+  git lfs migrate import --above 121B --include "*.md" && exit 1
+  git lfs migrate import --above 121B --exclude "*.txt" && exit 1
+  git lfs migrate import --above 121B --fixup && exit 1
+  true
+)
+end_test
+
 begin_test "migrate import (existing .gitattributes)"
 (
   set -e
 
   setup_local_branch_with_gitattrs
 
-  pwd
+  main="$(git rev-parse refs/heads/main)"
 
-  master="$(git rev-parse refs/heads/master)"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "$main:a.txt")")"
 
-  txt_master_oid="$(calc_oid "$(git cat-file -p "$master:a.txt")")"
+  git lfs migrate import --yes --include-ref=refs/heads/main --include="*.txt"
 
-  git lfs migrate import --yes --include-ref=refs/heads/master --include="*.txt"
+  assert_local_object "$txt_main_oid" "120"
 
-  assert_local_object "$txt_master_oid" "120"
+  main="$(git rev-parse refs/heads/main)"
+  prev="$(git rev-parse refs/heads/main^1)"
 
-  master="$(git rev-parse refs/heads/master)"
-  prev="$(git rev-parse refs/heads/master^1)"
-
-  diff -u <(git cat-file -p $master:.gitattributes) <(cat <<-EOF
+  diff -u <(git cat-file -p $main:.gitattributes) <(cat <<-EOF
 *.txt filter=lfs diff=lfs merge=lfs -text
 *.other filter=lfs diff=lfs merge=lfs -text
 EOF)
@@ -435,6 +530,161 @@ EOF)
 )
 end_test
 
+begin_test "migrate import (--exclude with existing .gitattributes)"
+(
+  set -e
+
+  setup_local_branch_with_gitattrs
+
+  main="$(git rev-parse refs/heads/main)"
+
+  txt_main_oid="$(calc_oid "$(git cat-file -p "$main:a.txt")")"
+
+  git lfs migrate import --yes --include-ref=refs/heads/main --include="*.txt" --exclude="*.bin"
+
+  assert_local_object "$txt_main_oid" "120"
+
+  main="$(git rev-parse refs/heads/main)"
+  prev="$(git rev-parse refs/heads/main^1)"
+
+  diff -u <(git cat-file -p $main:.gitattributes) <(cat <<-EOF
+*.txt filter=lfs diff=lfs merge=lfs -text
+*.other filter=lfs diff=lfs merge=lfs -text
+*.bin !text -filter -merge -diff
+EOF)
+
+  diff -u <(git cat-file -p $prev:.gitattributes) <(cat <<-EOF
+*.txt filter=lfs diff=lfs merge=lfs -text
+*.bin !text -filter -merge -diff
+EOF)
+)
+end_test
+
+begin_test "migrate import (existing .gitattributes with different permissions)"
+(
+  set -e
+
+  # Windows lacks POSIX permissions.
+  [ "$IS_WINDOWS" -eq 1 ] && exit 0
+
+  setup_local_branch_with_gitattrs 0755
+
+  main="$(git rev-parse refs/heads/main)"
+
+  txt_main_oid="$(calc_oid "$(git cat-file -p "$main:a.txt")")"
+
+  [ -x .gitattributes ]
+
+  git lfs migrate import --yes --include-ref=refs/heads/main --include="*.txt"
+
+  [ ! -x .gitattributes ]
+
+  assert_local_object "$txt_main_oid" "120"
+
+  main="$(git rev-parse refs/heads/main)"
+  prev="$(git rev-parse refs/heads/main^1)"
+
+  diff -u <(git cat-file -p $main:.gitattributes) <(cat <<-EOF
+*.txt filter=lfs diff=lfs merge=lfs -text
+*.other filter=lfs diff=lfs merge=lfs -text
+EOF
+  )
+
+  diff -u <(git cat-file -p $prev:.gitattributes) <(cat <<-EOF
+*.txt filter=lfs diff=lfs merge=lfs -text
+EOF
+  )
+
+  attrs_main_sha="$(git show $main:.gitattributes | git hash-object --stdin)"
+  txt_main_sha="$(git show $main:a.txt | git hash-object --stdin)"
+
+  diff -u <(git ls-tree $main) <(cat <<-EOF
+100644 blob $attrs_main_sha	.gitattributes
+100644 blob $txt_main_sha	a.txt
+EOF
+  )
+)
+end_test
+
+begin_test "migrate import (existing .gitattributes symlink)"
+(
+  set -e
+
+  setup_local_branch_with_gitattrs link
+
+  git lfs migrate import --yes --include-ref=refs/heads/main --include="*.txt" 2>&1 | tee migrate.log
+  if [ ${PIPESTATUS[0]} -eq 0 ]; then
+    echo >&2 "fatal: expected git lfs migrate import to fail, didn't"
+    exit 1
+  fi
+
+  grep "migrate: expected '.gitattributes' to be a file, got a symbolic link" migrate.log
+
+  main="$(git rev-parse refs/heads/main)"
+
+  attrs_main_sha="$(git show $main:.gitattributes | git hash-object --stdin)"
+
+  diff -u <(git ls-tree $main -- .gitattributes) <(cat <<-EOF
+120000 blob $attrs_main_sha	.gitattributes
+EOF
+  )
+)
+end_test
+
+begin_test "migrate import (identical contents, different permissions)"
+(
+  set -e
+
+  # Windows lacks POSIX permissions.
+  [ "$IS_WINDOWS" -eq 1 ] && exit 0
+
+  setup_multiple_local_branches
+  git checkout main
+
+  echo "foo" >foo.dat
+  git add .
+  git commit -m "add file"
+
+  chmod u+x foo.dat
+  git add .
+  git commit -m "make file executable"
+
+  # Verify we have executable permissions.
+  [ -x foo.dat ]
+
+  git lfs migrate import --everything --include="*.dat"
+
+  # Verify we have executable permissions.
+  [ -x foo.dat ]
+)
+end_test
+
+begin_test "migrate import (tags with same name as branches)"
+(
+  set -e
+
+  setup_multiple_local_branches
+  git checkout main
+
+  contents="hello"
+  oid=$(calc_oid "$contents")
+  printf "$contents" >hello.dat
+  git add .
+  git commit -m "add file"
+
+  git branch foo
+  git tag foo
+  git tag bar
+
+  git lfs migrate import --everything --include="*.dat"
+
+  [ "$(git rev-parse refs/heads/foo)" = "$(git rev-parse refs/tags/foo)" ]
+  [ "$(git rev-parse refs/heads/foo)" = "$(git rev-parse refs/tags/bar)" ]
+
+  assert_pointer "refs/heads/foo" hello.dat "$oid" 5
+)
+end_test
+
 begin_test "migrate import (bare repository)"
 (
   set -e
@@ -443,7 +693,7 @@ begin_test "migrate import (bare repository)"
   make_bare
 
   git lfs migrate import \
-    --include-ref=master
+    --include-ref=main
 )
 end_test
 
@@ -466,7 +716,7 @@ begin_test "migrate import (prefix include(s))"
 (
   set -e
 
-  includes="foo/bar/baz foo/**/baz/a.txt *.txt"
+  includes="foo/bar/baz/** foo/**/baz/a.txt *.txt"
   for include in $includes; do
     setup_single_local_branch_deep_trees
 
@@ -486,19 +736,19 @@ begin_test "migrate import (--everything)"
   set -e
 
   setup_multiple_local_branches
-  git checkout master
+  git checkout main
 
-  master_txt_oid="$(calc_oid "$(git cat-file -p :a.txt)")"
-  master_md_oid="$(calc_oid "$(git cat-file -p :a.md)")"
+  main_txt_oid="$(calc_oid "$(git cat-file -p :a.txt)")"
+  main_md_oid="$(calc_oid "$(git cat-file -p :a.md)")"
   feature_md_oid="$(calc_oid "$(git cat-file -p my-feature:a.md)")"
-  master_txt_size="$(git cat-file -p :a.txt | wc -c | awk '{ print $1 }')"
-  master_md_size="$(git cat-file -p :a.md | wc -c | awk '{ print $1 }')"
+  main_txt_size="$(git cat-file -p :a.txt | wc -c | awk '{ print $1 }')"
+  main_md_size="$(git cat-file -p :a.md | wc -c | awk '{ print $1 }')"
   feature_md_size="$(git cat-file -p my-feature:a.md | wc -c | awk '{ print $1 }')"
 
   git lfs migrate import --everything
 
-  assert_pointer "master" "a.txt" "$master_txt_oid" "$master_txt_size"
-  assert_pointer "master" "a.md" "$master_md_oid" "$master_md_size"
+  assert_pointer "main" "a.txt" "$main_txt_oid" "$main_txt_size"
+  assert_pointer "main" "a.md" "$main_md_oid" "$main_md_size"
   assert_pointer "my-feature" "a.md" "$feature_md_oid" "$feature_md_size"
 )
 end_test
@@ -524,8 +774,8 @@ begin_test "migrate import (--everything with args)"
 
   setup_multiple_local_branches
 
-  [ "$(git lfs migrate import --everything master 2>&1)" = \
-    "fatal: cannot use --everything with explicit reference arguments" ]
+  [ "$(git lfs migrate import --everything main 2>&1)" = \
+    "Cannot use --everything with explicit reference arguments" ]
 )
 end_test
 
@@ -535,8 +785,8 @@ begin_test "migrate import (--everything with --include-ref)"
 
   setup_multiple_local_branches
 
-  [ "$(git lfs migrate import --everything --include-ref=refs/heads/master 2>&1)" = \
-    "fatal: cannot use --everything with --include-ref or --exclude-ref" ]
+  [ "$(git lfs migrate import --everything --include-ref=refs/heads/main 2>&1)" = \
+    "Cannot use --everything with --include-ref or --exclude-ref" ]
 )
 end_test
 
@@ -546,8 +796,8 @@ begin_test "migrate import (--everything with --exclude-ref)"
 
   setup_multiple_local_branches
 
-  [ "$(git lfs migrate import --everything --exclude-ref=refs/heads/master 2>&1)" = \
-    "fatal: cannot use --everything with --include-ref or --exclude-ref" ]
+  [ "$(git lfs migrate import --everything --exclude-ref=refs/heads/main 2>&1)" = \
+    "Cannot use --everything with --include-ref or --exclude-ref" ]
 )
 end_test
 
@@ -557,19 +807,50 @@ begin_test "migrate import (--everything and --include with glob pattern)"
 
   setup_multiple_local_branches
 
-  md_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.md")")"
-  txt_master_oid="$(calc_oid "$(git cat-file -p "refs/heads/master:a.txt")")"
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
   md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
   txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
 
   git lfs migrate import --verbose --everything --include='*.[mM][dD]'
 
-  assert_pointer "refs/heads/master" "a.md" "$md_master_oid" "140"
+  assert_pointer "refs/heads/main" "a.md" "$md_main_oid" "140"
   assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "30"
 
-  assert_local_object "$md_master_oid" "140"
+  assert_local_object "$md_main_oid" "140"
   assert_local_object "$md_feature_oid" "30"
-  refute_local_object "$txt_master_oid"
+  refute_local_object "$txt_main_oid"
+  refute_local_object "$txt_feature_oid"
+)
+end_test
+
+begin_test "migrate import (--everything with tag pointing to tag)"
+(
+  set -e
+
+  setup_multiple_local_branches
+
+  md_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.md")")"
+  txt_main_oid="$(calc_oid "$(git cat-file -p "refs/heads/main:a.txt")")"
+  md_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.md")")"
+  txt_feature_oid="$(calc_oid "$(git cat-file -p "refs/heads/my-feature:a.txt")")"
+
+  git tag -a -m abc abc refs/heads/main
+  git tag -a -m def def refs/tags/abc
+
+  git lfs migrate import --verbose --everything --include='*.[mM][dD]'
+
+  assert_pointer "refs/heads/main" "a.md" "$md_main_oid" "140"
+  assert_pointer "refs/tags/abc" "a.md" "$md_main_oid" "140"
+  assert_pointer "refs/tags/def" "a.md" "$md_main_oid" "140"
+  assert_pointer "refs/heads/my-feature" "a.md" "$md_feature_oid" "30"
+
+  git tag --points-at refs/tags/abc | grep -q def
+  ! git tag --points-at refs/tags/def | grep -q abc
+
+  assert_local_object "$md_main_oid" "140"
+  assert_local_object "$md_feature_oid" "30"
+  refute_local_object "$txt_main_oid"
   refute_local_object "$txt_feature_oid"
 )
 end_test
@@ -585,7 +866,7 @@ begin_test "migrate import (nested sub-trees and --include with wildcard)"
 
   git lfs migrate import --include="**/*ar/**"
 
-  assert_pointer "refs/heads/master" "foo/bar/baz/a.txt" "$oid" "$size"
+  assert_pointer "refs/heads/main" "foo/bar/baz/a.txt" "$oid" "$size"
   assert_local_object "$oid" "$size"
 )
 end_test
@@ -608,7 +889,7 @@ begin_test "migrate import (handle copies of files)"
   # only import objects under "foo"
   git lfs migrate import --include="foo/**"
 
-  assert_pointer "refs/heads/master" "foo/bar/baz/a.txt" "$oid_tree" "$size"
+  assert_pointer "refs/heads/main" "foo/bar/baz/a.txt" "$oid_tree" "$size"
   assert_local_object "$oid_tree" "$size"
 
   # "a.txt" is not under "foo" and therefore should not be in LFS
@@ -617,6 +898,26 @@ begin_test "migrate import (handle copies of files)"
 )
 end_test
 
+begin_test "migrate import (filter matches files only)"
+(
+  set -e
+
+  setup_single_local_branch_same_file_tree_ext
+
+  txt_root_oid="$(calc_oid "$(git cat-file -p :a.txt)")"
+  txt_foo_oid="$(calc_oid "$(git cat-file -p :foo/a.txt)")"
+  md_bar_oid="$(calc_oid "$(git cat-file -p :bar.txt/b.md)")"
+  txt_bar_oid="$(calc_oid "$(git cat-file -p :bar.txt/b.txt)")"
+
+  git lfs migrate import --include="*.txt"
+
+  assert_local_object "$txt_root_oid" "120"
+  assert_local_object "$txt_foo_oid" "120"
+  assert_local_object "$txt_bar_oid" "120"
+  refute_local_object "$md_bar_oid"
+)
+end_test
+
 begin_test "migrate import (--object-map)"
 (
   set -e
@@ -644,7 +945,7 @@ begin_test "migrate import (--include with space)"
 
   git lfs migrate import --include "a file.txt"
 
-  assert_pointer "refs/heads/master" "a file.txt" "$oid" 50
+  assert_pointer "refs/heads/main" "a file.txt" "$oid" 50
   cat .gitattributes
   if [ 1 -ne "$(grep -c "a\[\[:space:\]\]file.txt" .gitattributes)" ]; then
     echo >&2 "fatal: expected \"a[[:space:]]file.txt\" to appear in .gitattributes"
@@ -666,7 +967,7 @@ begin_test "migrate import (handle symbolic link)"
 
   git lfs migrate import --include="*.txt"
 
-  assert_pointer "refs/heads/master" "a.txt" "$txt_oid" "120"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_oid" "120"
 
   assert_local_object "$txt_oid" "120"
   # "link.txt" is a symbolic link so it should be not in LFS
@@ -698,13 +999,30 @@ begin_test "migrate import (multiple remotes)"
 
   setup_multiple_remotes
 
-  original_master="$(git rev-parse master)"
+  original_main="$(git rev-parse main)"
 
   git lfs migrate import
 
-  migrated_master="$(git rev-parse master)"
+  migrated_main="$(git rev-parse main)"
 
-  assert_ref_unmoved "master" "$original_master" "$migrated_master"
+  assert_ref_unmoved "main" "$original_main" "$migrated_main"
+)
+end_test
+
+begin_test "migrate import (dirty copy, default negative answer)"
+(
+  set -e
+
+  setup_local_branch_with_dirty_copy
+
+  original_main="$(git rev-parse main)"
+
+  echo | git lfs migrate import --everything 2>&1 | tee migrate.log
+  grep "migrate: working copy must not be dirty" migrate.log
+
+  migrated_main="$(git rev-parse main)"
+
+  assert_ref_unmoved "main" "$original_main" "$migrated_main"
 )
 end_test
 
@@ -714,14 +1032,14 @@ begin_test "migrate import (dirty copy, negative answer)"
 
   setup_local_branch_with_dirty_copy
 
-  original_master="$(git rev-parse master)"
+  original_main="$(git rev-parse main)"
 
   echo "n" | git lfs migrate import --everything 2>&1 | tee migrate.log
   grep "migrate: working copy must not be dirty" migrate.log
 
-  migrated_master="$(git rev-parse master)"
+  migrated_main="$(git rev-parse main)"
 
-  assert_ref_unmoved "master" "$original_master" "$migrated_master"
+  assert_ref_unmoved "main" "$original_main" "$migrated_main"
 )
 end_test
 
@@ -731,7 +1049,7 @@ begin_test "migrate import (dirty copy, unknown then negative answer)"
 
   setup_local_branch_with_dirty_copy
 
-  original_master="$(git rev-parse master)"
+  original_main="$(git rev-parse main)"
 
   echo "x\nn" | git lfs migrate import --everything 2>&1 | tee migrate.log
 
@@ -741,9 +1059,9 @@ begin_test "migrate import (dirty copy, unknown then negative answer)"
     | wc -l | awk '{ print $1 }')" ]
   grep "migrate: working copy must not be dirty" migrate.log
 
-  migrated_master="$(git rev-parse master)"
+  migrated_main="$(git rev-parse main)"
 
-  assert_ref_unmoved "master" "$original_master" "$migrated_master"
+  assert_ref_unmoved "main" "$original_main" "$migrated_main"
 )
 end_test
 
@@ -759,7 +1077,7 @@ begin_test "migrate import (dirty copy, positive answer)"
   grep "migrate: changes in your working copy will be overridden ..." \
     migrate.log
 
-  assert_pointer "refs/heads/master" "a.txt" "$oid" "5"
+  assert_pointer "refs/heads/main" "a.txt" "$oid" "5"
   assert_local_object "$oid" "5"
 )
 end_test
@@ -776,8 +1094,8 @@ begin_test "migrate import (non-standard refs)"
 
   git lfs migrate import --everything
 
-  assert_pointer "refs/heads/master" "a.md" "$md_oid" "140"
-  assert_pointer "refs/heads/master" "a.txt" "$txt_oid" "120"
+  assert_pointer "refs/heads/main" "a.md" "$md_oid" "140"
+  assert_pointer "refs/heads/main" "a.txt" "$txt_oid" "120"
   assert_pointer "refs/pull/1/base" "a.md" "$md_oid" "140"
   assert_pointer "refs/pull/1/base" "a.txt" "$txt_oid" "120"
 
@@ -789,3 +1107,51 @@ begin_test "migrate import (non-standard refs)"
   assert_local_object "$md_feature_oid" "30"
 )
 end_test
+
+begin_test "migrate import (copied file)"
+(
+  set -e
+
+  setup_local_branch_with_copied_file
+
+  git lfs migrate import --above=1b
+
+  # Expect attributes for "/dir/a" and "/a"
+  if ! grep -q "^/dir/a.txt" ./.gitattributes || ! grep -q "^/a.txt" ./.gitattributes; then
+    exit 1
+  fi
+)
+end_test
+
+begin_test "migrate import (copied file with only a single path)"
+(
+  set -e
+
+  setup_local_branch_with_copied_file
+
+  oid="$(calc_oid "$(git cat-file -p :a.txt)")"
+
+  # Prevent MSYS from rewriting /a.txt into a Windows path.
+  MSYS_NO_PATHCONV=1 git lfs migrate import --include="/a.txt" --everything
+
+  # Expect attribute for only "/a.txt".
+  if grep -q "^/dir/a.txt" ./.gitattributes || ! grep -q "^/a.txt" ./.gitattributes; then
+    exit 1
+  fi
+  refute_pointer "refs/heads/main" "dir/a.txt" "$oid" 5
+)
+end_test
+
+begin_test "migrate import (filename special characters)"
+(
+  set -e
+  setup_local_branch_with_special_character_files
+  git lfs migrate import --above=1b
+  # Windows does not allow creation of files with '*', so expect 2 files, not 3
+  if [ "$IS_WINDOWS" -eq "1" ] ; then
+    test "$(git check-attr filter -- *.bin |grep lfs | wc -l)" -eq 2 || exit 1
+  else
+    test "$(git check-attr filter -- *.bin |grep lfs | wc -l)" -eq 3 || exit 1
+  fi
+)
+end_test
