@@ -3,7 +3,7 @@
 . "$(dirname "$0")/testlib.sh"
 
 # sets up the repos for the first few push tests. The passed argument is the
-# name of the repo to setup. The resuling repo will have a local file tracked
+# name of the repo to setup. The resulting repo will have a local file tracked
 # with LFS and committed, but not yet pushed to the remote
 push_repo_setup() {
   reponame="$1"
@@ -20,9 +20,9 @@ push_repo_setup() {
 begin_test "push with good ref"
 (
   set -e
-  push_repo_setup "push-master-branch-required"
+  push_repo_setup "push-main-branch-required"
 
-  git lfs push origin master
+  git lfs push origin main
 )
 end_test
 
@@ -33,8 +33,8 @@ begin_test "push with tracked ref"
   push_repo_setup "push-tracked-branch-required"
 
   git config push.default upstream
-  git config branch.master.merge refs/heads/tracked
-  git lfs push origin master
+  git config branch.main.merge refs/heads/tracked
+  git lfs push origin main
 )
 end_test
 
@@ -43,13 +43,13 @@ begin_test "push with bad ref"
   set -e
   push_repo_setup "push-other-branch-required"
 
-  git lfs push origin master 2>&1 | tee push.log
+  git lfs push origin main 2>&1 | tee push.log
   if [ "0" -eq "${PIPESTATUS[0]}" ]; then
     echo "expected command to fail"
     exit 1
   fi
 
-  grep 'batch response: Expected ref "refs/heads/other", got "refs/heads/master"' push.log
+  grep 'batch response: Expected ref "refs/heads/other", got "refs/heads/main"' push.log
 )
 end_test
 
@@ -60,12 +60,23 @@ begin_test "push with given remote, configured pushRemote"
 
   git remote add bad-remote "invalid-url"
 
-  git config branch.master.pushRemote bad-remote
+  git config branch.main.pushRemote bad-remote
 
   git lfs push --all origin
 )
 end_test
 
+begin_test "push via stdin with extra arguments"
+(
+  set -e
+
+  push_repo_setup "push-stdin-extra-args"
+
+  echo "main" | git lfs push origin --stdin --dry-run "another-ref" \
+    2>&1 | tee push.log
+  grep "Further command line arguments are ignored with --stdin" push.log
+)
+
 begin_test "push"
 (
   set -e
@@ -81,11 +92,11 @@ begin_test "push"
   git add .gitattributes a.dat
   git commit -m "add a.dat"
 
-  git lfs push --dry-run origin master 2>&1 | tee push.log
+  git lfs push --dry-run origin main 2>&1 | tee push.log
   grep "push 4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340 => a.dat" push.log
-  [ $(grep -c "push" push.log) -eq 1 ]
+  [ $(grep -c "^push " push.log) -eq 1 ]
 
-  git lfs push origin master 2>&1 | tee push.log
+  git lfs push origin main 2>&1 | tee push.log
   grep "Uploading LFS objects: 100% (1/1), 7 B" push.log
 
   git checkout -b push-b
@@ -96,14 +107,19 @@ begin_test "push"
   git lfs push --dry-run origin push-b 2>&1 | tee push.log
   grep "push 4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340 => a.dat" push.log
   grep "push 82be50ad35070a4ef3467a0a650c52d5b637035e7ad02c36652e59d01ba282b7 => b.dat" push.log
-  [ $(grep -c "push" < push.log) -eq 2 ]
+  [ $(grep -c "^push " < push.log) -eq 2 ]
+
+  printf "push-b\n\n" | git lfs push --dry-run origin --stdin 2>&1 | tee push.log
+  grep "push 4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340 => a.dat" push.log
+  grep "push 82be50ad35070a4ef3467a0a650c52d5b637035e7ad02c36652e59d01ba282b7 => b.dat" push.log
+  [ $(grep -c "^push " < push.log) -eq 2 ]
 
   # simulate remote ref
   mkdir -p .git/refs/remotes/origin
   git rev-parse HEAD > .git/refs/remotes/origin/HEAD
 
   git lfs push --dry-run origin push-b 2>&1 | tee push.log
-  [ $(grep -c "push" push.log) -eq 0 ]
+  [ $(grep -c "^push " push.log) -eq 0 ]
 
   rm -rf .git/refs/remotes
 
@@ -112,15 +128,15 @@ begin_test "push"
 )
 end_test
 
-# sets up the tests for the next few push --all tests
+# helper used by the next few push --all tests to set up their repos
 push_all_setup() {
   suffix="$1"
-  reponame="$(basename "$0" ".sh")-all"
+  reponame="$(basename "$0" ".sh")-all-$suffix"
   content1="initial"
   content2="update"
   content3="branch"
   content4="tagged"
-  content5="master"
+  content5="main"
   extracontent="extra"
   oid1=$(calc_oid "$content1")
   oid2=$(calc_oid "$content2")
@@ -129,10 +145,7 @@ push_all_setup() {
   oid5=$(calc_oid "$content5")
   extraoid=$(calc_oid "$extracontent")
 
-  # if the local repo exists, it has already been bootstrapped
-  [ -d "push-all" ] && exit 0
-
-  clone_repo "$reponame" "push-all"
+  clone_repo "$reponame" "push-all-$suffix"
   git config "lfs.$(repo_endpoint "$GITSERVER" "$reponame").locksverify" true
   git lfs track "*.dat"
 
@@ -158,7 +171,7 @@ push_all_setup() {
   },
   {
     \"CommitDate\":\"$(get_date -4m)\",
-    \"ParentBranches\":[\"master\"],
+    \"ParentBranches\":[\"main\"],
     \"Tags\":[\"tag\"],
     \"Files\":[
       {\"Filename\":\"file1.dat\",\"Size\":${#content4},\"Data\":\"$content4\"}
@@ -197,10 +210,10 @@ begin_test "push --all (no ref args)"
   grep "push $oid4 => file1.dat" push.log
   grep "push $oid5 => file1.dat" push.log
   grep "push $extraoid => file2.dat" push.log
-  [ $(grep -c "push" < push.log) -eq 6 ]
+  [ $(grep -c "^push " < push.log) -eq 6 ]
 
   git push --all origin 2>&1 | tee push.log
-  [ $(grep -c "Uploading LFS objects: 100% (6/6), 36 B" push.log) -eq 1 ]
+  [ $(grep -c "Uploading LFS objects: 100% (6/6)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix" "$oid1"
   assert_server_object "$reponame-$suffix" "$oid2"
   assert_server_object "$reponame-$suffix" "$oid3"
@@ -229,10 +242,10 @@ begin_test "push --all (no ref args)"
   grep "push $oid4 => file1.dat" push.log
   grep "push $oid5 => file1.dat" push.log
   grep "push $extraoid => file2.dat" push.log
-  [ $(grep -c "push" push.log) -eq 6 ]
+  [ $(grep -c "^push " push.log) -eq 6 ]
 
   git push --all origin 2>&1 | tee push.log
-  grep "Uploading LFS objects: 100% (6/6), 36 B" push.log
+  grep "Uploading LFS objects: 100% (6/6)" push.log
   assert_server_object "$reponame-$suffix-2" "$oid2"
   assert_server_object "$reponame-$suffix-2" "$oid3"
   assert_server_object "$reponame-$suffix-2" "$oid4"
@@ -251,14 +264,14 @@ begin_test "push --all (1 ref arg)"
   grep "push $oid1 => file1.dat" push.log
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid3 => file1.dat" push.log
-  [ $(grep -c "push" < push.log) -eq 3 ]
+  [ $(grep -c "^push " < push.log) -eq 3 ]
 
   git lfs push --all origin branch 2>&1 | tee push.log
-  grep "3 files" push.log
+  [ $(grep -c "Uploading LFS objects: 100% (3/3)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix" "$oid1"
   assert_server_object "$reponame-$suffix" "$oid2"
   assert_server_object "$reponame-$suffix" "$oid3"
-  refute_server_object "$reponame-$suffix" "$oid4"     # in master and the tag
+  refute_server_object "$reponame-$suffix" "$oid4"     # in main and the tag
   refute_server_object "$reponame-$suffix" "$oid5"
   refute_server_object "$reponame-$suffix" "$extraoid"
 
@@ -279,10 +292,10 @@ begin_test "push --all (1 ref arg)"
   grep "push $oid1 => file1.dat" push.log
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid3 => file1.dat" push.log
-  [ $(grep -c "push" push.log) -eq 3 ]
+  [ $(grep -c "^push " push.log) -eq 3 ]
 
-  git push --all origin branch 2>&1 | tee push.log
-  grep "5 files, 1 skipped" push.log # should be 5?
+  git push origin branch 2>&1 | tee push.log
+  [ $(grep -c "Uploading LFS objects: 100% (3/3)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix-2" "$oid2"
   assert_server_object "$reponame-$suffix-2" "$oid3"
   refute_server_object "$reponame-$suffix-2" "$oid4"
@@ -302,15 +315,22 @@ begin_test "push --all (multiple ref args)"
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid3 => file1.dat" push.log
   grep "push $oid4 => file1.dat" push.log
-  [ $(grep -c "push" push.log) -eq 4 ]
+  [ $(grep -c "^push " push.log) -eq 4 ]
+
+  printf "branch\ntag" | git lfs push --dry-run --all origin --stdin 2>&1 | tee push.log
+  grep "push $oid1 => file1.dat" push.log
+  grep "push $oid2 => file1.dat" push.log
+  grep "push $oid3 => file1.dat" push.log
+  grep "push $oid4 => file1.dat" push.log
+  [ $(grep -c "^push " push.log) -eq 4 ]
 
   git lfs push --all origin branch tag 2>&1 | tee push.log
-  grep "4 files" push.log
+  [ $(grep -c "Uploading LFS objects: 100% (4/4)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix" "$oid1"
   assert_server_object "$reponame-$suffix" "$oid2"
   assert_server_object "$reponame-$suffix" "$oid3"
   assert_server_object "$reponame-$suffix" "$oid4"
-  refute_server_object "$reponame-$suffix" "$oid5"     # only in master
+  refute_server_object "$reponame-$suffix" "$oid5"     # only in main
   refute_server_object "$reponame-$suffix" "$extraoid"
 
   echo "push while missing old objects locally"
@@ -331,10 +351,10 @@ begin_test "push --all (multiple ref args)"
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid3 => file1.dat" push.log
   grep "push $oid4 => file1.dat" push.log
-  [ $(grep -c "push" push.log) -eq 3 ]
+  [ $(grep -c "^push " push.log) -eq 4 ]
 
-  git push --all origin branch tag 2>&1 | tee push.log
-  grep "5 files, 1 skipped" push.log # should be 5?
+  git push origin branch refs/tags/tag 2>&1 | tee push.log
+  [ $(grep -c "Uploading LFS objects: 100% (4/4)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix-2" "$oid2"
   assert_server_object "$reponame-$suffix-2" "$oid3"
   assert_server_object "$reponame-$suffix-2" "$oid4"
@@ -349,16 +369,16 @@ begin_test "push --all (ref with deleted files)"
 
   push_all_setup "ref-with-deleted"
 
-  git lfs push --dry-run --all origin master 2>&1 | tee push.log
+  git lfs push --dry-run --all origin main 2>&1 | tee push.log
   grep "push $oid1 => file1.dat" push.log
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid4 => file1.dat" push.log
   grep "push $oid5 => file1.dat" push.log
   grep "push $extraoid => file2.dat" push.log
-  [ $(grep -c "push" push.log) -eq 5 ]
+  [ $(grep -c "^push " push.log) -eq 5 ]
 
-  git lfs push --all origin master 2>&1 | tee push.log
-  grep "5 files" push.log
+  git lfs push --all origin main 2>&1 | tee push.log
+  [ $(grep -c "Uploading LFS objects: 100% (5/5)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix" "$oid1"
   assert_server_object "$reponame-$suffix" "$oid2"
   refute_server_object "$reponame-$suffix" "$oid3" # only in the branch
@@ -379,16 +399,16 @@ begin_test "push --all (ref with deleted files)"
   rm ".git/lfs/objects/${oid1:0:2}/${oid1:2:2}/$oid1"
 
   # dry run doesn't change
-  git lfs push --dry-run --all origin master 2>&1 | tee push.log
+  git lfs push --dry-run --all origin main 2>&1 | tee push.log
   grep "push $oid1 => file1.dat" push.log
   grep "push $oid2 => file1.dat" push.log
   grep "push $oid4 => file1.dat" push.log
   grep "push $oid5 => file1.dat" push.log
   grep "push $extraoid => file2.dat" push.log
-  [ $(grep -c "push" push.log) -eq 5 ]
+  [ $(grep -c "^push " push.log) -eq 5 ]
 
-  git push --all origin master 2>&1 | tee push.log
-  grep "5 files, 1 skipped" push.log # should be 5?
+  git push origin main 2>&1 | tee push.log
+  [ $(grep -c "Uploading LFS objects: 100% (5/5)" push.log) -eq 1 ]
   assert_server_object "$reponame-$suffix-2" "$oid2"
   refute_server_object "$reponame-$suffix-2" "$oid3"
   assert_server_object "$reponame-$suffix-2" "$oid4"
@@ -429,6 +449,47 @@ begin_test "push object id(s)"
 )
 end_test
 
+begin_test "push object id(s) via stdin"
+(
+  set -e
+
+  reponame="$(basename "$0" ".sh")"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" repo3
+
+  git config "lfs.$(repo_endpoint "$GITSERVER" "$reponame").locksverify" true
+
+  git lfs track "*.dat"
+  echo "push a" > a.dat
+  git add .gitattributes a.dat
+  git commit -m "add a.dat"
+
+  echo "" | git lfs push --object-id origin --stdin --dry-run \
+    2>&1 | tee push.log
+  grep "At least one object ID must be supplied with --object-id" push.log
+
+  echo "4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340" | \
+    git lfs push --object-id origin --stdin --dry-run "c0ffee" \
+    2>&1 | tee push.log
+  grep "Further command line arguments are ignored with --stdin" push.log
+
+  echo "4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340" | \
+    git lfs push --object-id origin --stdin --dry-run \
+    2>&1 | tee push.log
+  grep "push 4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340 =>" push.log
+
+  echo "push b" > b.dat
+  git add b.dat
+  git commit -m "add b.dat"
+
+  printf "4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340\n82be50ad35070a4ef3467a0a650c52d5b637035e7ad02c36652e59d01ba282b7\n\n" | \
+    git lfs push --object-id origin --stdin --dry-run \
+    2>&1 | tee push.log
+  grep "push 4c48d2a6991c9895bcddcf027e1e4907280bcf21975492b1afbade396d6a3340 =>" push.log
+  grep "push 82be50ad35070a4ef3467a0a650c52d5b637035e7ad02c36652e59d01ba282b7 =>" push.log
+)
+end_test
+
 begin_test "push modified files"
 (
   set -e
@@ -469,14 +530,14 @@ begin_test "push modified files"
   },
   {
     \"CommitDate\":\"$(get_date -1m)\",
-    \"ParentBranches\":[\"master\"],
+    \"ParentBranches\":[\"main\"],
     \"Files\":[
       {\"Filename\":\"file1.dat\",\"Size\":${#content3}, \"Data\":\"$content3\"},
       {\"Filename\":\"file2.dat\",\"Size\":${#content4}, \"Data\":\"$content4\"}]
   }
   ]" | lfstest-testutils addcommits
 
-  git lfs push origin master
+  git lfs push origin main
   git lfs push origin other_branch
   assert_server_object "$reponame" "$oid1"
   assert_server_object "$reponame" "$oid2"
@@ -534,7 +595,7 @@ begin_test "push ambiguous branch name"
       {\"Filename\":\"file4.dat\",\"Size\":${#content[3]}, \"Data\":\"${content[3]}\"}]
   },
   {
-    \"ParentBranches\":[\"master\"],
+    \"ParentBranches\":[\"main\"],
     \"CommitDate\":\"$(get_date -1d)\",
     \"Files\":[
       {\"Filename\":\"file1.dat\",\"Size\":${#content[4]}, \"Data\":\"${content[4]}\"}]
@@ -544,8 +605,8 @@ begin_test "push ambiguous branch name"
   # create tag with same name as branch
   git tag ambiguous
 
-  # lfs push master, should work
-  git lfs push origin master
+  # lfs push main, should work
+  git lfs push origin main
 
   # push ambiguous, does not fail since lfs scans git with sha, not ref name
   git lfs push origin ambiguous
@@ -563,19 +624,19 @@ begin_test "push (retry with expired actions)"
   git lfs track "*.dat"
   contents="return-expired-action"
   contents_oid="$(calc_oid "$contents")"
-  contents_size="$(printf "$contents" | wc -c | awk '{ print $1 }')"
-  printf "$contents" > a.dat
+  contents_size="$(printf "%s" "$contents" | wc -c | awk '{ print $1 }')"
+  printf "%s" "$contents" > a.dat
   git add .gitattributes a.dat
 
   git commit -m "add a.dat, .gitattributes" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "2 files changed" commit.log
   grep "create mode 100644 a.dat" commit.log
   grep "create mode 100644 .gitattributes" commit.log
 
-  GIT_TRACE=1 git push origin master 2>&1 | tee push.log
+  GIT_TRACE=1 git push origin main 2>&1 | tee push.log
 
-  expected="enqueue retry #1 for \"$contents_oid\" (size: $contents_size): LFS: tq: action \"upload\" expires at"
+  expected="enqueue retry #1 after 0.25s for \"$contents_oid\" (size: $contents_size): LFS: action \"upload\" expires at"
 
   grep "$expected" push.log
   grep "Uploading LFS objects: 100% (1/1), 21 B" push.log
@@ -596,17 +657,17 @@ begin_test "push to raw remote url"
   contents="raw"
   contents_oid=$(calc_oid "$contents")
 
-  printf "$contents" > raw.dat
+  printf "%s" "$contents" > raw.dat
   git add raw.dat .gitattributes
   git commit -m "add" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "2 files changed" commit.log
   grep "create mode 100644 raw.dat" commit.log
   grep "create mode 100644 .gitattributes" commit.log
 
   refute_server_object push-raw "$contents_oid"
 
-  git lfs push $GITSERVER/push-raw master
+  git lfs push $GITSERVER/push-raw main
 
   assert_server_object push-raw "$contents_oid"
 )
@@ -622,17 +683,17 @@ begin_test "push (with invalid object size)"
 
   git lfs track "*.dat"
   contents="return-invalid-size"
-  printf "$contents" > a.dat
+  printf "%s" "$contents" > a.dat
 
   git add a.dat .gitattributes
   git commit -m "add a.dat, .gitattributes" 2>&1 | tee commit.log
-  grep "master (root-commit)" commit.log
+  grep "main (root-commit)" commit.log
   grep "2 files changed" commit.log
   grep "create mode 100644 a.dat" commit.log
   grep "create mode 100644 .gitattributes" commit.log
 
   set +e
-  git push origin master 2>&1 2> push.log
+  git push origin main 2>&1 2> push.log
   res="$?"
   set -e
 
@@ -658,12 +719,220 @@ begin_test "push with deprecated _links"
 
   contents="send-deprecated-links"
   contents_oid="$(calc_oid "$contents")"
-  printf "$contents" > a.dat
+  printf "%s" "$contents" > a.dat
   git add a.dat
   git commit -m "add a.dat"
 
-  git push origin master
+  git push origin main
 
   assert_server_object "$reponame" "$contents_oid"
 )
 end_test
+
+begin_test "push with invalid pushInsteadof"
+(
+  set -e
+
+  push_repo_setup "push-invalid-pushinsteadof"
+
+  # set pushInsteadOf to rewrite the href of uploading LFS object.
+  git config url."$GITSERVER/storage/invalid".pushInsteadOf "$GITSERVER/storage/"
+  # Enable href rewriting explicitly.
+  git config lfs.transfer.enablehrefrewrite true
+
+  set +e
+  git lfs push origin main > push.log 2>&1
+  res=$?
+
+  set -e
+  [ "$res" = "2" ]
+
+  # check rewritten href is used to upload LFS object.
+  grep "LFS: Authorization error: $GITSERVER/storage/invalid" push.log
+
+  # lfs-push succeed after unsetting enableHrefRewrite config
+  git config --unset lfs.transfer.enablehrefrewrite
+  git lfs push origin main
+)
+end_test
+
+begin_test 'push with data the server already has'
+(
+  set -e
+
+  reponame="push-server-data"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git lfs track "*.dat"
+  git add .gitattributes
+  git commit -m "initial commit"
+
+  contents="abc123"
+  contents_oid="$(calc_oid "$contents")"
+  printf "%s" "$contents" > a.dat
+  git add a.dat
+  git commit -m "add a.dat"
+
+  git push origin main
+
+  assert_server_object "$reponame" "$contents_oid"
+
+  git checkout -b side
+
+  # Use a different file name for the second file; otherwise, this test
+  # unexpectedly passes with the old code, since we fail to notice that the
+  # object we run through the clean filter is not the object we wanted.
+  contents2="def456"
+  contents2_oid="$(calc_oid "$contents2")"
+  printf "%s" "$contents2" > b.dat
+  git add b.dat
+  git commit -m "add b.dat"
+
+  # We remove the original object. The server already has this.
+  delete_local_object "$contents_oid"
+
+  # We use the URL so that we cannot take advantage of the existing "origin/*"
+  # refs that we know the server must have. We will traverse the entire history
+  # for this push, and we should not fail because the server already has the
+  # object we deleted above.
+  git push "$(git config remote.origin.url)" side
+
+  assert_server_object "$reponame" "$contents2_oid"
+)
+end_test
+
+begin_test 'push with multiple refs and data the server already has'
+(
+  set -e
+
+  reponame="push-multi-ref-server-data"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git lfs track "*.dat"
+  git add .gitattributes
+  git commit -m "initial commit"
+
+  contents="abc123"
+  contents_oid="$(calc_oid "$contents")"
+  printf "%s" "$contents" > a.dat
+  git add a.dat
+  git commit -m "add a.dat"
+
+  git push origin main
+
+  assert_server_object "$reponame" "$contents_oid"
+
+  contents2="def456"
+  contents2_oid="$(calc_oid "$contents2")"
+  printf "%s" "$contents2" > b.dat
+  git add b.dat
+  git commit -m "add b.dat"
+
+  # Create a tag.  Normally this would cause the entire history to be traversed
+  # since it's a new ref, but we no longer do that since we're pushing multiple
+  # refs.
+  git tag -m v1.0.0 -a v1.0.0
+
+  # We remove the original object. The server already has this.
+  delete_local_object "$contents_oid"
+
+  # We use the URL so that we cannot take advantage of the existing "origin/*"
+  # refs that we know the server must have.
+  GIT_TRACE=1 GIT_TRANSFER_TRACE=1 GIT_CURL_VERBOSE=1 \
+    git push "$(git config remote.origin.url)" main v1.0.0 2>&1 | tee push.log
+
+  # We should not find a batch request for the object which is in the earlier
+  # version of main, since we know the remote side has it.
+  [ "$(grep -c "$contents_oid" push.log)" = 0 ]
+
+  # Yet we should have pushed the new object successfully.
+  assert_server_object "$reponame" "$contents2_oid"
+)
+end_test
+
+begin_test "push custom reference"
+(
+  set -e
+
+  reponame="push-custom-reference"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git lfs track "*.dat"
+  # generate content we'll use
+  content="filecontent"
+  oid=$(calc_oid "$content")
+
+  echo "[
+  {
+    \"CommitDate\":\"$(get_date -6m)\",
+    \"Files\":[
+      {\"Filename\":\"file.dat\",\"Size\":${#content}, \"Data\":\"$content\"}]
+  }
+  ]" | lfstest-testutils addcommits
+
+  # Create and try pushing a reference in a nonstandard namespace, that is,
+  # outside of refs/heads, refs/tags, and refs/remotes.
+  git update-ref refs/custom/remote/heads/main refs/heads/main
+
+  git lfs push origin refs/custom/remote/heads/main
+  assert_server_object "$reponame" "$oid"
+)
+end_test
+
+begin_test "push --object-id (invalid value)"
+(
+  set -e
+
+  push_all_setup "push-invalid-oid"
+
+  git lfs push --object-id origin '' 2>&1 | tee push.log
+  git lfs push --object-id origin "${oid1:0:3}" 2>&1 | tee -a push.log
+
+  [ "$(grep -c 'too short object ID' push.log)" -eq 2 ]
+)
+end_test
+
+begin_test "storage upload with compression"
+(
+  set -e
+
+  reponame="storage-compress"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" storage-compress
+
+  contents="storage-compress"
+  oid="$(calc_oid "$contents")"
+  printf "%s" "$contents" > a.dat
+
+  git lfs track "*.dat"
+  git add .gitattributes a.dat
+  git commit -m "initial commit"
+
+  GIT_CURL_VERBOSE=1 git push origin main | tee push.log
+  assert_server_object "$reponame" "$oid"
+
+  pushd ..
+    git \
+      -c "filter.lfs.process=" \
+      -c "filter.lfs.smudge=cat" \
+      -c "filter.lfs.required=false" \
+      clone "$GITSERVER/$reponame" "$reponame-assert"
+
+    cd "$reponame-assert"
+
+    git config credential.helper lfstest
+
+    GIT_TRACE=1 git lfs pull origin main 2>&1 | tee pull.log
+    if [ "0" -ne "${PIPESTATUS[0]}" ]; then
+      echo >&2 "fatal: expected \`git lfs pull origin main\` to succeed ..."
+      exit 1
+    fi
+
+    grep "decompressed gzipped response" pull.log
+    assert_local_object "$oid" "${#contents}"
+  popd
+)
+end_test
