@@ -45,6 +45,36 @@ begin_test "track"
   grep "* text=auto" .gitattributes
   grep "diff=csharp" .gitattributes
   grep "*.jpg" .gitattributes
+
+  echo "*.gif -filter -text" >> a/b/.gitattributes
+  echo "*.mov -filter -text" >> a/b/.gitattributes
+
+  git lfs track | tee track.log
+  tail -n 3 track.log | head -n 1 | grep "Listing excluded patterns"
+  tail -n 3 track.log | grep "*.gif ($(native_path_escaped "a/b/.gitattributes"))"
+  tail -n 3 track.log | grep "*.mov ($(native_path_escaped "a/b/.gitattributes"))"
+)
+end_test
+
+begin_test "track --no-excluded"
+(
+  set -e
+
+  reponame="track_no_excluded"
+  mkdir "$reponame"
+  cd "$reponame"
+  git init
+
+  mkdir -p a/b .git/info
+
+  echo "*.mov filter=lfs -text" > .git/info/attributes
+  echo "*.gif filter=lfs -text" > a/.gitattributes
+  echo "*.png filter=lfs -text" > a/b/.gitattributes
+  echo "*.gif -filter -text" >> a/b/.gitattributes
+  echo "*.mov -filter=lfs -text" >> a/b/.gitattributes
+
+  git lfs track --no-excluded | tee track.log
+  ! grep "Listing excluded patterns" track.log
 )
 end_test
 
@@ -61,7 +91,7 @@ begin_test "track --verbose"
   git add foo.dat
 
   git lfs track --verbose "foo.dat" 2>&1 > track.log
-  grep "touching \"foo.dat\"" track.log
+  grep "Touching \"foo.dat\"" track.log
 )
 end_test
 
@@ -79,7 +109,7 @@ begin_test "track --dry-run"
 
   git lfs track --dry-run "foo.dat" 2>&1 > track.log
   grep "Tracking \"foo.dat\"" track.log
-  grep "Git LFS: touching \"foo.dat\"" track.log
+  grep "Touching \"foo.dat\"" track.log
 
   git status --porcelain 2>&1 > status.log
   grep "A  foo.dat" status.log
@@ -103,8 +133,8 @@ begin_test "track directory"
   git add foo\ bar
   git commit -am "add foo bar"
 
-  assert_pointer "master" "foo bar/a" "87428fc522803d31065e7bce3cf03fe475096631e5e07bbd7a0fde60c4cf25c7" 2
-  assert_pointer "master" "foo bar/b" "0263829989b6fd954f72baaf2fc64bc2e2f01d692d4de72986ea808f6e99813f" 2
+  assert_pointer "main" "foo bar/a" "87428fc522803d31065e7bce3cf03fe475096631e5e07bbd7a0fde60c4cf25c7" 2
+  assert_pointer "main" "foo bar/b" "0263829989b6fd954f72baaf2fc64bc2e2f01d692d4de72986ea808f6e99813f" 2
 )
 end_test
 
@@ -182,9 +212,12 @@ begin_test "track with autocrlf=input"
   [ "*.mov filter=lfs -text" = "$(cat .gitattributes)" ]
 
   git lfs track "*.gif"
-  expected="*.mov filter=lfs -text^M$
-*.gif filter=lfs diff=lfs merge=lfs -text^M$"
-  [ "$expected" = "$(cat -e .gitattributes)" ]
+  if [ $IS_WINDOWS -eq 1 ]
+  then
+      cat -e .gitattributes | grep '\^M\$'
+  else
+      cat -e .gitattributes | grep -v '\^M'
+  fi
 )
 end_test
 
@@ -338,7 +371,7 @@ begin_test "track blocklisted files by name"
   git add .gitattributes
 
   git lfs track .gitattributes 2>&1 > track.log
-  grep "Pattern .gitattributes matches forbidden file .gitattributes" track.log
+  grep "Pattern '.gitattributes' matches forbidden file '.gitattributes'" track.log
 )
 end_test
 
@@ -355,7 +388,7 @@ begin_test "track blocklisted files with glob"
   git add .gitattributes
 
   git lfs track ".git*" 2>&1 > track.log
-  grep "Pattern .git\* matches forbidden file" track.log
+  grep "Pattern '.git\*' matches forbidden file" track.log
 )
 end_test
 
@@ -413,6 +446,7 @@ begin_test "track lockable read-only/read-write"
   mkdir subfolder
   echo "sub blah blah" > subfolder/test.bin
   echo "sub foo bar" > subfolder/test.dat
+  git add *.bin *.dat subfolder
   # should start writeable
   assert_file_writeable test.bin
   assert_file_writeable test.dat
@@ -610,7 +644,7 @@ begin_test "track: escaped pattern in .gitattributes"
 
   [ "Tracking \"$filename\"" = "$(git lfs track "$filename")" ]
   [ "\"$filename\" already supported" = "$(git lfs track "$filename")" ]
-  
+
   #changing flags should track the file again
   [ "Tracking \"$filename\"" = "$(git lfs track -l "$filename")" ]
 
@@ -620,3 +654,101 @@ begin_test "track: escaped pattern in .gitattributes"
   fi
 )
 end_test
+
+begin_test "track: escaped glob pattern in .gitattributes"
+(
+  set -e
+
+  # None of these characters are valid in the Win32 subsystem.
+  [ "$IS_WINDOWS" -eq 1 ] && exit 0
+
+  reponame="track-escaped-glob"
+  git init "$reponame"
+  cd "$reponame"
+
+  filename='*[foo]bar?.txt'
+  contents='I need escaping'
+  contents_oid=$(calc_oid "$contents")
+
+  git lfs track --filename "$filename"
+  git lfs track --filename "$filename" | grep 'already supported'
+  git add .
+  cat .gitattributes
+
+  printf "%s" "$contents" > "$filename"
+  git add .
+  git commit -m 'Add unusually named file'
+
+  # If Git understood our escaping, we'll have a pointer. Otherwise, we won't.
+  assert_pointer "main" "$filename" "$contents_oid" 15
+)
+end_test
+
+begin_test "track: escaped glob pattern with spaces in .gitattributes"
+(
+  set -e
+
+  # None of these characters are valid in the Win32 subsystem.
+  [ "$IS_WINDOWS" -eq 1 ] && exit 0
+
+  reponame="track-escaped-glob"
+  git init "$reponame"
+  cd "$reponame"
+
+  filename="*[foo] bar?.txt"
+  contents='I need escaping'
+  contents_oid=$(calc_oid "$contents")
+
+  git lfs track --filename "$filename"
+  git add .
+  cat .gitattributes
+
+  printf "%s" "$contents" > "$filename"
+  git add .
+  git commit -m 'Add unusually named file'
+
+  # If Git understood our escaping, we'll have a pointer. Otherwise, we won't.
+  assert_pointer "main" "$filename" "$contents_oid" 15
+)
+end_test
+
+begin_test "--json output"
+(
+  set -e
+
+  reponame="track-json"
+  git init "$reponame"
+  cd "$reponame"
+
+  git lfs track '*.dat'
+  git lfs track --lockable '*.bin'
+  echo 'a.dat !filter' >>.gitattributes
+
+  git lfs track --json > actual
+  cat >expected <<-EOF
+{
+ "patterns": [
+  {
+   "pattern": "*.dat",
+   "source": ".gitattributes",
+   "lockable": false,
+   "tracked": true
+  },
+  {
+   "pattern": "*.bin",
+   "source": ".gitattributes",
+   "lockable": true,
+   "tracked": true
+  },
+  {
+   "pattern": "a.dat",
+   "source": ".gitattributes",
+   "lockable": false,
+   "tracked": false
+  }
+ ]
+}
+EOF
+  diff -u actual expected
+)
+end_test
