@@ -3,11 +3,12 @@
 package tools
 
 import (
-	"bufio"
 	"encoding/hex"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
+	"os/user"
 	"path/filepath"
 	"runtime"
 	"strconv"
@@ -15,7 +16,9 @@ import (
 	"sync"
 	"sync/atomic"
 
-	"github.com/git-lfs/git-lfs/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/filepathfilter"
+	"github.com/git-lfs/git-lfs/v3/tr"
 )
 
 // FileOrDirExists determines if a file/dir exists, returns IsDir() results too.
@@ -58,7 +61,7 @@ func ResolveSymlinks(path string) string {
 		return path
 	}
 
-	if resolved, err := filepath.EvalSymlinks(path); err == nil {
+	if resolved, err := CanonicalizeSystemPath(path); err == nil {
 		return resolved
 	}
 	return path
@@ -74,12 +77,12 @@ func RenameFileCopyPermissions(srcfile, destfile string) error {
 		return err
 	} else {
 		if err := os.Chmod(srcfile, info.Mode()); err != nil {
-			return fmt.Errorf("can't set filemode on file %q: %v", srcfile, err)
+			return errors.New(tr.Tr.Get("can't set filemode on file %q: %v", srcfile, err))
 		}
 	}
 
-	if err := os.Rename(srcfile, destfile); err != nil {
-		return fmt.Errorf("cannot replace %q with %q: %v", destfile, srcfile, err)
+	if err := RobustRename(srcfile, destfile); err != nil {
+		return errors.New(tr.Tr.Get("cannot replace %q with %q: %v", destfile, srcfile, err))
 	}
 	return nil
 }
@@ -113,6 +116,104 @@ func CleanPaths(paths, delim string) (cleaned []string) {
 	return cleaned
 }
 
+// repositoryPermissionFetcher is an interface that matches the configuration
+// object and can be used to fetch repository permissions.
+type repositoryPermissionFetcher interface {
+	RepositoryPermissions(executable bool) os.FileMode
+}
+
+// MkdirAll makes a directory and any intervening directories with the
+// permissions specified by the core.sharedRepository setting.
+func MkdirAll(path string, config repositoryPermissionFetcher) error {
+	umask := 0777 & ^config.RepositoryPermissions(true)
+	return doWithUmask(int(umask), func() error {
+		return os.MkdirAll(path, config.RepositoryPermissions(true))
+	})
+}
+
+var (
+	// currentUser is a wrapper over user.Current(), but instead uses the
+	// value of os.Getenv("HOME") for the returned *user.User's "HomeDir"
+	// member.
+	currentUser func() (*user.User, error) = func() (*user.User, error) {
+		u, err := user.Current()
+		if err != nil {
+			return nil, err
+		}
+		u.HomeDir = os.Getenv("HOME")
+		return u, nil
+	}
+	lookupUser       func(who string) (*user.User, error) = user.Lookup
+	lookupConfigHome func() string                        = func() string {
+		return os.Getenv("XDG_CONFIG_HOME")
+	}
+)
+
+// ExpandPath returns a copy of path with any references to the current user's
+// home directory (spelled "~"), or a named user's home directory (spelled
+// "~user") in the path, sanitized to the calling filesystem's path separator
+// preference.
+//
+// If the "expand" argument is given as true, the resolved path to the named
+// user's home directory will expanded with filepath.EvalSymlinks.
+//
+// If either the current or named user does not have a home directory, an error
+// will be returned.
+//
+// Otherwise, the error returned will be nil, and the string returned will be
+// the expanded path.
+func ExpandPath(path string, expand bool) (string, error) {
+	if len(path) == 0 || path[0] != '~' {
+		return path, nil
+	}
+
+	var username string
+	if slash := strings.Index(path[1:], "/"); slash > -1 {
+		username = path[1 : slash+1]
+	} else {
+		username = path[1:]
+	}
+
+	var (
+		who *user.User
+		err error
+	)
+	if len(username) == 0 {
+		who, err = currentUser()
+	} else {
+		who, err = lookupUser(username)
+	}
+
+	if err != nil {
+		return "", errors.Wrapf(err, tr.Tr.Get("could not find user %s", username))
+	}
+
+	homedir := who.HomeDir
+	if expand {
+		homedir, err = filepath.EvalSymlinks(homedir)
+		if err != nil {
+			return "", errors.Wrapf(err, tr.Tr.Get("cannot eval symlinks for %s", homedir))
+		}
+	}
+	return filepath.Join(homedir, path[len(username)+1:]), nil
+}
+
+// ExpandConfigPath returns a copy of path expanded as with ExpandPath.  If the
+// path is empty, the default path is looked up inside $XDG_CONFIG_HOME, or
+// ~/.config if that is not set.
+func ExpandConfigPath(path, defaultPath string) (string, error) {
+	if path != "" {
+		return ExpandPath(path, false)
+	}
+
+	configHome := lookupConfigHome()
+	if configHome != "" {
+		return filepath.Join(configHome, defaultPath), nil
+	}
+
+	return ExpandPath(fmt.Sprintf("~/.config/%s", defaultPath), false)
+}
+
 // VerifyFileHash reads a file and verifies whether the SHA is correct
 // Returns an error if there is a problem
 func VerifyFileHash(oid, path string) error {
@@ -130,7 +231,7 @@ func VerifyFileHash(oid, path string) error {
 
 	calcOid := hex.EncodeToString(h.Sum(nil))
 	if calcOid != oid {
-		return fmt.Errorf("File %q has an invalid hash %s, expected %s", path, calcOid, oid)
+		return errors.New(tr.Tr.Get("file %q has an invalid hash %s, expected %s", path, calcOid, oid))
 	}
 
 	return nil
@@ -139,20 +240,24 @@ func VerifyFileHash(oid, path string) error {
 // FastWalkCallback is the signature for the callback given to FastWalkGitRepo()
 type FastWalkCallback func(parentDir string, info os.FileInfo, err error)
 
-// FastWalkGitRepo is a more optimal implementation of filepath.Walk for a Git
+// FastWalkDir is a more optimal implementation of filepath.Walk for a Git
 // repo. The callback guaranteed to be called sequentially. The function returns
 // once all files and errors have triggered callbacks.
 // It differs in the following ways:
-//  * Uses goroutines to parallelise large dirs and descent into subdirs
-//  * Does not provide sorted output; parents will always be before children but
-//    there are no other guarantees. Use parentDir argument in the callback to
-//    determine absolute path rather than tracking it yourself
-//  * Automatically ignores any .git directories
-//  * Respects .gitignore contents and skips ignored files/dirs
+//   - Uses goroutines to parallelise large dirs and descent into subdirs
+//   - Does not provide sorted output; parents will always be before children but
+//     there are no other guarantees. Use parentDir argument in the callback to
+//     determine absolute path rather than tracking it yourself
+//   - Automatically ignores any .git directories
 //
 // rootDir - Absolute path to the top of the repository working directory
-func FastWalkGitRepo(rootDir string, cb FastWalkCallback) {
-	walker := fastWalkWithExcludeFiles(rootDir, ".gitignore")
+func FastWalkDir(rootDir string, cb FastWalkCallback) {
+	fastWalkCallback(fastWalkWithExcludeFiles(rootDir), cb)
+}
+
+// fastWalkCallback calls the FastWalkCallback "cb" for all files found by the
+// given *fastWalker, "walker".
+func fastWalkCallback(walker *fastWalker, cb FastWalkCallback) {
 	for file := range walker.ch {
 		cb(file.ParentDir, file.Info, file.Err)
 	}
@@ -168,23 +273,21 @@ type fastWalkInfo struct {
 }
 
 type fastWalker struct {
-	rootDir         string
-	excludeFilename string
-	ch              chan fastWalkInfo
-	limit           int32
-	cur             *int32
-	wg              *sync.WaitGroup
+	rootDir string
+	ch      chan fastWalkInfo
+	limit   int32
+	cur     *int32
+	wg      *sync.WaitGroup
 }
 
 // fastWalkWithExcludeFiles walks the contents of a dir, respecting
-// include/exclude patterns and also loading new exlude patterns from files
-// named excludeFilename in directories walked
+// include/exclude patterns.
 //
 // rootDir - Absolute path to the top of the repository working directory
-func fastWalkWithExcludeFiles(rootDir, excludeFilename string) *fastWalker {
+func fastWalkWithExcludeFiles(rootDir string) *fastWalker {
 	excludePaths := []filepathfilter.Pattern{
-		filepathfilter.NewPattern(".git"),
-		filepathfilter.NewPattern("**/.git"),
+		filepathfilter.NewPattern(".git", filepathfilter.GitIgnore),
+		filepathfilter.NewPattern("**/.git", filepathfilter.GitIgnore),
 	}
 
 	limit, _ := strconv.Atoi(os.Getenv("LFS_FASTWALK_LIMIT"))
@@ -194,15 +297,16 @@ func fastWalkWithExcludeFiles(rootDir, excludeFilename string) *fastWalker {
 
 	c := int32(0)
 	w := &fastWalker{
-		rootDir:         rootDir,
-		excludeFilename: excludeFilename,
-		limit:           int32(limit),
-		cur:             &c,
-		ch:              make(chan fastWalkInfo, 256),
-		wg:              &sync.WaitGroup{},
+		rootDir: rootDir,
+		limit:   int32(limit),
+		cur:     &c,
+		ch:      make(chan fastWalkInfo, 256),
+		wg:      &sync.WaitGroup{},
 	}
 
 	go func() {
+		defer w.Wait()
+
 		dirFi, err := os.Stat(w.rootDir)
 		if err != nil {
 			w.ch <- fastWalkInfo{Err: err}
@@ -210,17 +314,13 @@ func fastWalkWithExcludeFiles(rootDir, excludeFilename string) *fastWalker {
 		}
 
 		w.Walk(true, "", dirFi, excludePaths)
-		w.Wait()
 	}()
 	return w
 }
 
-// Walk is the main recursive implementation of fast walk.
-// Sends the file/dir and any contents to the channel so long as it passes the
-// include/exclude filter. If a dir, parses any excludeFilename found and updates
-// the excludePaths with its content before (parallel) recursing into contents
-// Also splits large directories into multiple goroutines.
-// Increments waitg.Add(1) for each new goroutine launched internally
+// Walk is the main recursive implementation of fast walk.  Sends the file/dir
+// and any contents to the channel so long as it passes the include/exclude
+// filter.  Increments waitg.Add(1) for each new goroutine launched internally
 //
 // workDir - Relative path inside the repository
 func (w *fastWalker) Walk(isRoot bool, workDir string, itemFi os.FileInfo,
@@ -235,6 +335,15 @@ func (w *fastWalker) Walk(isRoot bool, workDir string, itemFi os.FileInfo,
 		fullPath = join(parentWorkDir, itemFi.Name())
 	}
 
+	if !isRoot && itemFi.IsDir() {
+		// If this directory has a .git directory or file in it, then
+		// this is a submodule, and we should not recurse into it.
+		_, err := os.Stat(filepath.Join(fullPath, ".git"))
+		if err == nil {
+			return
+		}
+	}
+
 	workPath := join(workDir, itemFi.Name())
 	if !filepathfilter.NewFromPatterns(nil, excludePaths).Allows(workPath) {
 		return
@@ -252,15 +361,6 @@ func (w *fastWalker) Walk(isRoot bool, workDir string, itemFi os.FileInfo,
 		childWorkDir = join(workDir, itemFi.Name())
 	}
 
-	if len(w.excludeFilename) > 0 {
-		possibleExcludeFile := join(fullPath, w.excludeFilename)
-		var err error
-		excludePaths, err = loadExcludeFilename(possibleExcludeFile, childWorkDir, excludePaths)
-		if err != nil {
-			w.ch <- fastWalkInfo{Err: err}
-		}
-	}
-
 	// The absolute optimal way to scan would be File.Readdirnames but we
 	// still need the Stat() to know whether something is a dir, so use
 	// File.Readdir instead. Means we can provide os.FileInfo to callers like
@@ -309,51 +409,6 @@ func (w *fastWalker) Wait() {
 	close(w.ch)
 }
 
-// loadExcludeFilename reads the given file in gitignore format and returns a
-// revised array of exclude paths if there are any changes.
-// If any changes are made a copy of the array is taken so the original is not
-// modified
-func loadExcludeFilename(filename, workDir string, excludePaths []filepathfilter.Pattern) ([]filepathfilter.Pattern, error) {
-	f, err := os.OpenFile(filename, os.O_RDONLY, 0644)
-	if err != nil {
-		if os.IsNotExist(err) {
-			return excludePaths, nil
-		}
-		return excludePaths, err
-	}
-	defer f.Close()
-
-	retPaths := excludePaths
-	modified := false
-
-	scanner := bufio.NewScanner(f)
-	for scanner.Scan() {
-		line := strings.TrimSpace(scanner.Text())
-		// Skip blanks, comments and negations (not supported right now)
-		if len(line) == 0 || strings.HasPrefix(line, "#") || strings.HasPrefix(line, "!") {
-			continue
-		}
-
-		if !modified {
-			// copy on write
-			retPaths = make([]filepathfilter.Pattern, len(excludePaths))
-			copy(retPaths, excludePaths)
-			modified = true
-		}
-
-		path := line
-		// Add pattern in context if exclude has separator, or no wildcard
-		// Allow for both styles of separator at this point
-		if strings.ContainsAny(path, "/\\") ||
-			!strings.Contains(path, "*") {
-			path = join(workDir, line)
-		}
-		retPaths = append(retPaths, filepathfilter.NewPattern(path))
-	}
-
-	return retPaths, nil
-}
-
 func join(paths ...string) string {
 	ne := make([]string, 0, len(paths))
 
@@ -393,3 +448,58 @@ func SetFileWriteFlag(path string, writeEnabled bool) error {
 	}
 	return os.Chmod(path, os.FileMode(mode))
 }
+
+// TempFile creates a temporary file in specified directory with proper permissions for the repository.
+// On success, it returns an open, non-nil *os.File, and the caller is responsible
+// for closing and/or removing it.  On failure, the temporary file is
+// automatically cleaned up and an error returned.
+//
+// This function is designed to handle only temporary files that will be renamed
+// into place later somewhere within the Git repository.
+func TempFile(dir, pattern string, cfg repositoryPermissionFetcher) (*os.File, error) {
+	tmp, err := ioutil.TempFile(dir, pattern)
+	if err != nil {
+		return nil, err
+	}
+
+	perms := cfg.RepositoryPermissions(false)
+	err = os.Chmod(tmp.Name(), perms)
+	if err != nil {
+		tmp.Close()
+		os.Remove(tmp.Name())
+		return nil, err
+	}
+	return tmp, nil
+}
+
+// ExecutablePermissions takes a set of Unix permissions (which may or may not
+// have the executable bits set) and maps them into a set of permissions in
+// which the executable bits are set, using the same technique as Git does.
+func ExecutablePermissions(perms os.FileMode) os.FileMode {
+	// Copy read bits to executable bits.
+	return perms | ((perms & 0444) >> 2)
+}
+
+// CanonicalizePath takes a path and produces a canonical absolute path,
+// performing any OS- or environment-specific path transformations (within the
+// limitations of the Go standard library).  If the path is empty, it returns
+// the empty path with no error.  If missingOk is true, then if the
+// canonicalized path does not exist, an absolute path is given instead.
+func CanonicalizePath(path string, missingOk bool) (string, error) {
+	path, err := TranslateCygwinPath(path)
+	if err != nil {
+		return "", err
+	}
+	if len(path) > 0 {
+		path, err := filepath.Abs(path)
+		if err != nil {
+			return "", err
+		}
+		result, err := CanonicalizeSystemPath(path)
+		if err != nil && os.IsNotExist(err) && missingOk {
+			return path, nil
+		}
+		return result, err
+	}
+	return "", nil
+}
