@@ -5,20 +5,24 @@ import (
 	"os"
 	"sort"
 	"sync"
+	"time"
 
-	"github.com/git-lfs/git-lfs/errors"
-	"github.com/git-lfs/git-lfs/git"
-	"github.com/git-lfs/git-lfs/lfsapi"
-	"github.com/git-lfs/git-lfs/tools"
+	"github.com/git-lfs/git-lfs/v3/errors"
+	"github.com/git-lfs/git-lfs/v3/git"
+	"github.com/git-lfs/git-lfs/v3/lfshttp"
+	"github.com/git-lfs/git-lfs/v3/tools"
+	"github.com/git-lfs/git-lfs/v3/tr"
 	"github.com/rubyist/tracerx"
 )
 
 const (
 	defaultBatchSize = 100
+	baseRetryDelayMs = 250
 )
 
 type retryCounter struct {
-	MaxRetries int `git:"lfs.transfer.maxretries"`
+	MaxRetries    int
+	MaxRetryDelay int
 
 	// cmu guards count
 	cmu sync.Mutex
@@ -26,26 +30,23 @@ type retryCounter struct {
 	count map[string]int
 }
 
-// newRetryCounter instantiates a new *retryCounter. It parses the gitconfig
-// value: `lfs.transfer.maxretries`, and falls back to defaultMaxRetries if none
-// was provided.
-//
-// If it encountered an error in Unmarshaling the *config.Configuration, it will
-// be returned, otherwise nil.
+// newRetryCounter instantiates a new *retryCounter.
 func newRetryCounter() *retryCounter {
 	return &retryCounter{
-		MaxRetries: defaultMaxRetries,
-		count:      make(map[string]int),
+		MaxRetries:    defaultMaxRetries,
+		MaxRetryDelay: defaultMaxRetryDelay,
+		count:         make(map[string]int),
 	}
 }
 
-// Increment increments the number of retries for a given OID. It is safe to
-// call across multiple goroutines.
-func (r *retryCounter) Increment(oid string) {
+// Increment increments the number of retries for a given OID and returns the
+// new value. It is safe to call across multiple goroutines.
+func (r *retryCounter) Increment(oid string) int {
 	r.cmu.Lock()
 	defer r.cmu.Unlock()
 
 	r.count[oid]++
+	return r.count[oid]
 }
 
 // CountFor returns the current number of retries for a given OID. It is safe to
@@ -64,6 +65,22 @@ func (r *retryCounter) CanRetry(oid string) (int, bool) {
 	return count, count < r.MaxRetries
 }
 
+// ReadyTime returns the time from now when the current retry can occur or the
+// zero time if the retry can occur immediately.
+func (r *retryCounter) ReadyTime(oid string) time.Time {
+	count := r.CountFor(oid)
+	if count < 1 {
+		return time.Time{}
+	}
+
+	maxDelayMs := 1000 * uint64(r.MaxRetryDelay)
+	delay := uint64(baseRetryDelayMs) * (1 << uint(count-1))
+	if delay == 0 || delay > maxDelayMs {
+		delay = maxDelayMs
+	}
+	return time.Now().Add(time.Duration(delay) * time.Millisecond)
+}
+
 // batch implements the sort.Interface interface and enables sorting on a slice
 // of `*Transfer`s by object size.
 //
@@ -77,19 +94,42 @@ type batch []*objectTuple
 // Concat concatenates two batches together, returning a single, clamped batch as
 // "left", and the remainder of elements as "right". If the union of the
 // receiver and "other" has cardinality less than "size", "right" will be
-// returned as nil.
-func (b batch) Concat(other batch, size int) (left, right batch) {
+// returned as nil. Any object tuple that is not currently able to be retried
+// (ie Retry-After response), will also go into the right batch. Also, when object(s)
+// are returned that are rate-limited, return the minimum duration required to wait until
+// a object is ready.
+func (b batch) Concat(other batch, size int) (left, right batch, minWait time.Duration) {
 	u := batch(append(b, other...))
-	if len(u) <= size {
-		return u, nil
+	for _, ot := range u {
+		if time.Now().After(ot.ReadyTime) {
+			// The current time is past the time the object should
+			// be available.
+			left = append(left, ot)
+		} else {
+			// The time hasn't passed for the object.
+			right = append(right, ot)
+			wait := time.Until(ot.ReadyTime)
+			if minWait == 0 {
+				minWait = wait
+			} else if wait < minWait {
+				minWait = wait
+			}
+		}
 	}
-	return u[:size], u[size:]
+	if len(left) <= size {
+		// If the size of left fits the given size limit, return with no adjustments.
+		return left, right, minWait
+	}
+	// If left is too large, trip left up to size and append the rest to right.
+	right = append(right, left[size:]...)
+	left = left[:size]
+	return left, right, minWait
 }
 
 func (b batch) ToTransfers() []*Transfer {
 	transfers := make([]*Transfer, 0, len(b))
 	for _, t := range b {
-		transfers = append(transfers, &Transfer{Oid: t.Oid, Size: t.Size})
+		transfers = append(transfers, &Transfer{Oid: t.Oid, Size: t.Size, Missing: t.Missing})
 	}
 	return transfers
 }
@@ -98,6 +138,43 @@ func (b batch) Len() int           { return len(b) }
 func (b batch) Less(i, j int) bool { return b[i].Size < b[j].Size }
 func (b batch) Swap(i, j int)      { b[i], b[j] = b[j], b[i] }
 
+type abortableWaitGroup struct {
+	wq      sync.WaitGroup
+	counter int
+	mu      sync.Mutex
+}
+
+func newAbortableWaitGroup() *abortableWaitGroup {
+	return &abortableWaitGroup{}
+}
+
+func (q *abortableWaitGroup) Add(delta int) {
+	q.mu.Lock()
+	defer q.mu.Unlock()
+
+	q.counter += delta
+	q.wq.Add(delta)
+}
+
+func (q *abortableWaitGroup) Done() {
+	q.mu.Lock()
+	defer q.mu.Unlock()
+
+	q.counter -= 1
+	q.wq.Done()
+}
+
+func (q *abortableWaitGroup) Abort() {
+	q.mu.Lock()
+	defer q.mu.Unlock()
+
+	q.wq.Add(-q.counter)
+}
+
+func (q *abortableWaitGroup) Wait() {
+	q.wq.Wait()
+}
+
 // TransferQueue organises the wider process of uploading and downloading,
 // including calling the API, passing the actual transfer request to transfer
 // adapters, and dealing with progress, errors and retries.
@@ -125,7 +202,7 @@ type TransferQueue struct {
 	// wait is used to keep track of pending transfers. It is incremented
 	// once per unique OID on Add(), and is decremented when that transfer
 	// is marked as completed or failed, but not retried.
-	wait     sync.WaitGroup
+	wait     *abortableWaitGroup
 	manifest *Manifest
 	rc       *retryCounter
 
@@ -166,14 +243,17 @@ func (s *objects) First() *objectTuple {
 type objectTuple struct {
 	Name, Path, Oid string
 	Size            int64
+	Missing         bool
+	ReadyTime       time.Time
 }
 
 func (o *objectTuple) ToTransfer() *Transfer {
 	return &Transfer{
-		Name: o.Name,
-		Path: o.Path,
-		Oid:  o.Oid,
-		Size: o.Size,
+		Name:    o.Name,
+		Path:    o.Path,
+		Oid:     o.Oid,
+		Size:    o.Size,
+		Missing: o.Missing,
 	}
 }
 
@@ -222,6 +302,7 @@ func NewTransferQueue(dir Direction, manifest *Manifest, remote string, options
 		trMutex:   &sync.Mutex{},
 		manifest:  manifest,
 		rc:        newRetryCounter(),
+		wait:      newAbortableWaitGroup(),
 	}
 
 	for _, opt := range options {
@@ -229,7 +310,8 @@ func NewTransferQueue(dir Direction, manifest *Manifest, remote string, options
 	}
 
 	q.rc.MaxRetries = q.manifest.maxRetries
-	q.client.MaxRetries = q.manifest.maxRetries
+	q.rc.MaxRetryDelay = q.manifest.maxRetryDelay
+	q.client.SetMaxRetries(q.manifest.maxRetries)
 
 	if q.batchSize <= 0 {
 		q.batchSize = defaultBatchSize
@@ -258,12 +340,18 @@ func NewTransferQueue(dir Direction, manifest *Manifest, remote string, options
 //
 // Only one file will be transferred to/from the Path element of the first
 // transfer.
-func (q *TransferQueue) Add(name, path, oid string, size int64) {
+func (q *TransferQueue) Add(name, path, oid string, size int64, missing bool, err error) {
+	if err != nil {
+		q.errorc <- err
+		return
+	}
+
 	t := &objectTuple{
-		Name: name,
-		Path: path,
-		Oid:  oid,
-		Size: size,
+		Name:    name,
+		Path:    path,
+		Oid:     oid,
+		Size:    size,
+		Missing: missing,
 	}
 
 	if objs := q.remember(t); len(objs.objects) > 1 {
@@ -310,22 +398,22 @@ func (q *TransferQueue) remember(t *objectTuple) objects {
 // collectBatches collects batches in a loop, prioritizing failed items from the
 // previous before adding new items. The process works as follows:
 //
-//   1. Create a new batch, of size `q.batchSize`, and containing no items
-//   2. While the batch contains less items than `q.batchSize` AND the channel
-//      is open, read one item from the `q.incoming` channel.
-//      a. If the read was a channel close, go to step 4.
-//      b. If the read was a transferable item, go to step 3.
-//   3. Append the item to the batch.
-//   4. Sort the batch by descending object size, make a batch API call, send
-//      the items to the `*adapterBase`.
-//   5. In a separate goroutine, process the worker results, incrementing and
-//      appending retries if possible. On the main goroutine, accept new items
-//      into "pending".
-//   6. Concat() the "next" and "pending" batches such that no more items than
-//      the maximum allowed per batch are in next, and the rest are in pending.
-//   7. If the `q.incoming` channel is open, go to step 2.
-//   8. If the next batch is empty AND the `q.incoming` channel is closed,
-//      terminate immediately.
+//  1. Create a new batch, of size `q.batchSize`, and containing no items
+//  2. While the batch contains less items than `q.batchSize` AND the channel
+//     is open, read one item from the `q.incoming` channel.
+//     a. If the read was a channel close, go to step 4.
+//     b. If the read was a transferable item, go to step 3.
+//  3. Append the item to the batch.
+//  4. Sort the batch by descending object size, make a batch API call, send
+//     the items to the `*adapterBase`.
+//  5. In a separate goroutine, process the worker results, incrementing and
+//     appending retries if possible. On the main goroutine, accept new items
+//     into "pending".
+//  6. Concat() the "next" and "pending" batches such that no more items than
+//     the maximum allowed per batch are in next, and the rest are in pending.
+//  7. If the `q.incoming` channel is open, go to step 2.
+//  8. If the next batch is empty AND the `q.incoming` channel is closed,
+//     terminate immediately.
 //
 // collectBatches runs in its own goroutine.
 func (q *TransferQueue) collectBatches() {
@@ -353,31 +441,46 @@ func (q *TransferQueue) collectBatches() {
 		done := make(chan struct{})
 
 		var retries batch
+		var err error
 
 		go func() {
-			var err error
+			defer close(done)
+
+			if len(next) == 0 {
+				return
+			}
 
 			retries, err = q.enqueueAndCollectRetriesFor(next)
 			if err != nil {
 				q.errorc <- err
 			}
-
-			close(done)
 		}()
 
 		var collected batch
 		collected, closing = q.collectPendingUntil(done)
 
+		// If we've encountered a serious error here, abort immediately;
+		// don't process further batches.  Abort the wait queue so that
+		// we don't deadlock waiting for objects to complete when they
+		// never will.
+		if err != nil && !errors.IsRetriableError(err) {
+			q.wait.Abort()
+			break
+		}
+
 		// Ensure the next batch is filled with, in order:
 		//
 		// - retries from the previous batch,
 		// - new additions that were enqueued behind retries, &
 		// - items collected while the batch was processing.
-		next, pending = retries.Concat(append(pending, collected...), q.batchSize)
-
-		if closing && len(next) == 0 {
-			// If len(next) == 0, there are no items in "pending",
-			// and it is safe to exit.
+		var minWaitTime time.Duration
+		next, pending, minWaitTime = retries.Concat(append(pending, collected...), q.batchSize)
+		if len(next) == 0 && len(pending) != 0 {
+			// There are some pending that could not be queued.
+			// Wait the requested time before resuming loop.
+			time.Sleep(minWaitTime)
+		} else if len(next) == 0 && len(pending) == 0 && closing {
+			// There are no items remaining, it is safe to break
 			break
 		}
 	}
@@ -407,7 +510,7 @@ func (q *TransferQueue) collectPendingUntil(done <-chan struct{}) (pending batch
 
 // enqueueAndCollectRetriesFor makes a Batch API call and returns a "next" batch
 // containing all of the objects that failed from the previous batch and had
-// retries availale to them.
+// retries available to them.
 //
 // If an error was encountered while making the API request, _all_ of the items
 // from the previous batch (that have retries available to them) will be
@@ -419,13 +522,31 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 	next := q.makeBatch()
 	tracerx.Printf("tq: sending batch of size %d", len(batch))
 
+	enqueueRetry := func(t *objectTuple, err error, readyTime *time.Time) {
+		count := q.rc.Increment(t.Oid)
+
+		if readyTime == nil {
+			t.ReadyTime = q.rc.ReadyTime(t.Oid)
+		} else {
+			t.ReadyTime = *readyTime
+		}
+		delay := time.Until(t.ReadyTime).Seconds()
+
+		var errMsg string
+		if err != nil {
+			errMsg = fmt.Sprintf(": %s", err)
+		}
+		tracerx.Printf("tq: enqueue retry #%d after %.2fs for %q (size: %d)%s", count, delay, t.Oid, t.Size, errMsg)
+		next = append(next, t)
+	}
+
 	q.meter.Pause()
 	var bRes *BatchResponse
 	if q.manifest.standaloneTransferAgent != "" {
 		// Trust the external transfer agent can do everything by itself.
 		objects := make([]*Transfer, 0, len(batch))
 		for _, t := range batch {
-			objects = append(objects, &Transfer{Oid: t.Oid, Size: t.Size, Path: t.Path})
+			objects = append(objects, &Transfer{Oid: t.Oid, Size: t.Size, Path: t.Path, Missing: t.Missing})
 		}
 		bRes = &BatchResponse{
 			Objects:             objects,
@@ -437,21 +558,30 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 		var err error
 		bRes, err = Batch(q.manifest, q.direction, q.remote, q.ref, batch.ToTransfers())
 		if err != nil {
+			var hasNonScheduledErrors = false
 			// If there was an error making the batch API call, mark all of
 			// the objects for retry, and return them along with the error
 			// that was encountered. If any of the objects couldn't be
 			// retried, they will be marked as failed.
 			for _, t := range batch {
 				if q.canRetryObject(t.Oid, err) {
-					q.rc.Increment(t.Oid)
-
-					next = append(next, t)
+					hasNonScheduledErrors = true
+					enqueueRetry(t, err, nil)
+				} else if readyTime, canRetry := q.canRetryObjectLater(t.Oid, err); canRetry {
+					enqueueRetry(t, err, &readyTime)
 				} else {
+					hasNonScheduledErrors = true
 					q.wait.Done()
 				}
 			}
 
-			return next, err
+			// Only return error and mark operation as failure if at least one object
+			// was not enqueued for retrial at a later point.
+			if hasNonScheduledErrors {
+				return next, errors.NewRetriableError(err)
+			} else {
+				return next, nil
+			}
 		}
 	}
 
@@ -459,6 +589,22 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 		return next, nil
 	}
 
+	// We check first that all of the objects we want to upload are present,
+	// and abort if any are missing. We'll never have any objects marked as
+	// missing except possibly on upload, so just skip iterating over the
+	// objects in that case.
+	if q.direction == Upload {
+		for _, o := range bRes.Objects {
+			// If the server already has the object, the list of
+			// actions will be empty. It's fine if the file is
+			// missing in that case, since we don't need to upload
+			// it.
+			if o.Missing && len(o.Actions) != 0 {
+				return nil, errors.New(tr.Tr.Get("Unable to find source for object %v (try running `git lfs fetch --all`)", o.Oid))
+			}
+		}
+	}
+
 	q.useAdapter(bRes.TransferAdapterName)
 	q.meter.Start()
 
@@ -481,7 +627,7 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 			// Transfer object, then we give up on the
 			// transfer by telling the progress meter to
 			// skip the number of bytes in "o".
-			q.errorc <- errors.Errorf("[%v] The server returned an unknown OID.", o.Oid)
+			q.errorc <- errors.Errorf(tr.Tr.Get("[%v] The server returned an unknown OID.", o.Oid))
 
 			q.Skip(o.Size)
 			q.wait.Done()
@@ -491,13 +637,8 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 			tr := newTransfer(o, objects.First().Name, objects.First().Path)
 
 			if a, err := tr.Rel(q.direction.String()); err != nil {
-				// XXX(taylor): duplication
 				if q.canRetryObject(tr.Oid, err) {
-					q.rc.Increment(tr.Oid)
-					count := q.rc.CountFor(tr.Oid)
-
-					tracerx.Printf("tq: enqueue retry #%d for %q (size: %d): %s", count, tr.Oid, tr.Size, err)
-					next = append(next, objects.First())
+					enqueueRetry(objects.First(), err, nil)
 				} else {
 					q.errorc <- errors.Errorf("[%v] %v", tr.Name, err)
 
@@ -516,12 +657,7 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 
 	retries := q.addToAdapter(bRes.endpoint, toTransfer)
 	for t := range retries {
-		q.rc.Increment(t.Oid)
-		count := q.rc.CountFor(t.Oid)
-
-		tracerx.Printf("tq: enqueue retry #%d for %q (size: %d)", count, t.Oid, t.Size)
-
-		next = append(next, t)
+		enqueueRetry(t, nil, nil)
 	}
 
 	return next, nil
@@ -537,7 +673,7 @@ func (q *TransferQueue) makeBatch() batch { return make(batch, 0, q.batchSize) }
 // closed.
 //
 // addToAdapter returns immediately, and does not block.
-func (q *TransferQueue) addToAdapter(e lfsapi.Endpoint, pending []*Transfer) <-chan *objectTuple {
+func (q *TransferQueue) addToAdapter(e lfshttp.Endpoint, pending []*Transfer) <-chan *objectTuple {
 	retries := make(chan *objectTuple, len(pending))
 
 	if err := q.ensureAdapterBegun(e); err != nil {
@@ -587,7 +723,7 @@ func (q *TransferQueue) partitionTransfers(transfers []*Transfer) (present []*Tr
 		var err error
 
 		if t.Size < 0 {
-			err = errors.Errorf("Git LFS: object %q has invalid size (got: %d)", t.Oid, t.Size)
+			err = errors.Errorf(tr.Tr.Get("object %q has invalid size (got: %d)", t.Oid, t.Size))
 		} else {
 			fd, serr := os.Stat(t.Path)
 			if serr != nil {
@@ -637,8 +773,23 @@ func (q *TransferQueue) handleTransferResult(
 	if res.Error != nil {
 		// If there was an error encountered when processing the
 		// transfer (res.Transfer), handle the error as is appropriate:
+		if readyTime, canRetry := q.canRetryObjectLater(oid, res.Error); canRetry {
+			// If the object can't be retried now, but can be
+			// after a certain period of time, send it to
+			// the retry channel with a time when it's ready.
+			tracerx.Printf("tq: retrying object %s after %s seconds.", oid, time.Until(readyTime).Seconds())
+			q.trMutex.Lock()
+			objects, ok := q.transfers[oid]
+			q.trMutex.Unlock()
 
-		if q.canRetryObject(oid, res.Error) {
+			if ok {
+				t := objects.First()
+				t.ReadyTime = readyTime
+				retries <- t
+			} else {
+				q.errorc <- res.Error
+			}
+		} else if q.canRetryObject(oid, res.Error) {
 			// If the object can be retried, send it on the retries
 			// channel, where it will be read at the call-site and
 			// its retry count will be incremented.
@@ -729,7 +880,7 @@ func (q *TransferQueue) Skip(size int64) {
 	q.meter.Skip(size)
 }
 
-func (q *TransferQueue) ensureAdapterBegun(e lfsapi.Endpoint) error {
+func (q *TransferQueue) ensureAdapterBegun(e lfshttp.Endpoint) error {
 	q.adapterInitMutex.Lock()
 	defer q.adapterInitMutex.Unlock()
 
@@ -760,12 +911,9 @@ func (q *TransferQueue) ensureAdapterBegun(e lfsapi.Endpoint) error {
 	return nil
 }
 
-func (q *TransferQueue) toAdapterCfg(e lfsapi.Endpoint) AdapterConfig {
+func (q *TransferQueue) toAdapterCfg(e lfshttp.Endpoint) AdapterConfig {
 	apiClient := q.manifest.APIClient()
 	concurrency := q.manifest.ConcurrentTransfers()
-	if apiClient.Endpoints.AccessFor(e.Url) == lfsapi.NTLMAccess {
-		concurrency = 1
-	}
 
 	return &adapterConfig{
 		concurrentTransfers: concurrency,
@@ -774,17 +922,6 @@ func (q *TransferQueue) toAdapterCfg(e lfsapi.Endpoint) AdapterConfig {
 	}
 }
 
-var (
-	// contentTypeWarning is the message printed when a server returns an
-	// HTTP 422 at the end of a push.
-	contentTypeWarning = []string{
-		"Uploading failed due to unsupported Content-Type header(s).",
-		"Consider disabling Content-Type detection with:",
-		"",
-		"  $ git config lfs.contenttype false",
-	}
-)
-
 // Wait waits for the queue to finish processing all transfers. Once Wait is
 // called, Add will no longer add transfers to the queue. Any failed
 // transfers will be automatically retried once.
@@ -804,10 +941,15 @@ func (q *TransferQueue) Wait() {
 	q.meter.Flush()
 	q.errorwait.Wait()
 
+	if q.manifest.sshTransfer != nil {
+		q.manifest.sshTransfer.Shutdown()
+	}
+
 	if q.unsupportedContentType {
-		for _, line := range contentTypeWarning {
-			fmt.Fprintf(os.Stderr, "info: %s\n", line)
-		}
+		fmt.Fprintln(os.Stderr, tr.Tr.Get(`info: Uploading failed due to unsupported Content-Type header(s).
+info: Consider disabling Content-Type detection with:
+info:
+info:   $ git config lfs.contenttype false`))
 	}
 }
 
@@ -843,6 +985,12 @@ func (q *TransferQueue) canRetry(err error) bool {
 	return errors.IsRetriableError(err)
 }
 
+// canRetryLater returns the number of seconds until an error can be retried and if the error
+// is a delayed-retriable error.
+func (q *TransferQueue) canRetryLater(err error) (time.Time, bool) {
+	return errors.IsRetriableLaterError(err)
+}
+
 // canRetryObject returns whether the given error is retriable for the object
 // given by "oid". If the an OID has met its retry limit, then it will not be
 // able to be retried again. If so, canRetryObject returns whether or not that
@@ -856,6 +1004,15 @@ func (q *TransferQueue) canRetryObject(oid string, err error) bool {
 	return q.canRetry(err)
 }
 
+func (q *TransferQueue) canRetryObjectLater(oid string, err error) (time.Time, bool) {
+	if count, ok := q.rc.CanRetry(oid); !ok {
+		tracerx.Printf("tq: refusing to retry %q, too many retries (%d)", oid, count)
+		return time.Time{}, false
+	}
+
+	return q.canRetryLater(err)
+}
+
 // Errors returns any errors encountered during transfer.
 func (q *TransferQueue) Errors() []error {
 	return q.errors
